/**********************************************************************
 *
 *   freedoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"
#include "announcement.h"

/** -> result
 **
 ** @param    announcement   announcement
 **
 ** @result   name of the announcement
 **/
string
to_string(Announcement const announcement) noexcept
{
  switch (announcement) {
  case Announcement::noannouncement:
    return "no announcement";
    (void)_("Announcement::no announcement");
  case Announcement::no120:
    return "no 120";
    (void)_("Announcement::no 120");
  case Announcement::no90:
    return "no 90";
    (void)_("Announcement::no 90");
  case Announcement::no60:
    return "no 60";
    (void)_("Announcement::no 60");
  case Announcement::no30:
    return "no 30";
    (void)_("Announcement::no 30");
  case Announcement::no0:
    return "no 0";
    (void)_("Announcement::no 0");
  case Announcement::reply:
    return "reply";
    (void)_("Announcement::reply");
  case Announcement::no120_reply:
    return "no 120 reply";
    (void)_("Announcement::no 120 reply");
  case Announcement::no90_reply:
    return "no 90 reply";
    (void)_("Announcement::no 90 reply");
  case Announcement::no60_reply:
    return "no 60 reply";
    (void)_("Announcement::no 60 reply");
  case Announcement::no30_reply:
    return "no 30 reply";
    (void)_("Announcement::no 30 reply");
  case Announcement::no0_reply:
    return "no 0 reply";
    (void)_("Announcement::no 0 reply");
  } // switch(announcement)

  DEBUG_ASSERTION(false,
                  "::to_string(Announcement):\n"
                  "  announcement not found: " << static_cast<int>(announcement)
                 );

  return "";
} // string to_string(Announcement announcement)

/** -> result
 **
 ** @param    announcement   announcement
 **
 ** @return   translation of the announcement
 **/
string
gettext(Announcement const announcement)
{
  return gettext("Announcement::" + to_string(announcement));
}

/** writes the announcement into the stream
 **
 ** @param    ostr           output stream
 ** @param    announcement   announcement
 **
 ** @return   output stream
 **/
ostream&
operator<<(ostream& ostr, Announcement const announcement)
{ return (ostr << to_string(announcement)); }

/** -> result
 **
 ** @param    name   announcement name
 **
 ** @return   the announcement with name 'name'
 **/
Announcement
announcement_from_string(string const& name)
{
  for (auto a : announcement_list)
    if (name == to_string(a))
      return a;

  throw ReadException("unknown announcement '" + name + "'");
  return Announcement::noannouncement;
} // Announcement announcement_from_string(string name)

/** @param    announcement
 **
 ** @return   whether the announcement is noannouncement
 **/
bool
operator!(Announcement announcement) noexcept
{ return (announcement == Announcement::noannouncement); }

/** @param    lhs   left operand
 ** @param    rhs   right operand
 **
 ** @return   lhs && rhs
 **/
bool
operator&&(bool const lhs, Announcement const rhs) noexcept
{ return (lhs && (rhs != Announcement::noannouncement)); }

/** @param    lhs   left operand
 ** @param    rhs   right operand
 **
 ** @return   lhs && rhs
 **/
bool
operator&&(Announcement const lhs, bool const rhs) noexcept
{ return ((lhs != Announcement::noannouncement) && rhs); }

/** @param    lhs   left operand
 ** @param    rhs   right operand
 **
 ** @return   lhs || rhs
 **/
bool
operator||(bool const lhs, Announcement const rhs) noexcept
{ return (lhs || (rhs != Announcement::noannouncement)); }

/** @param    lhs   left operand
 ** @param    rhs   right operand
 **
 ** @return   lhs || rhs
 **/
bool
operator||(Announcement const lhs, bool const rhs) noexcept
{ return ((lhs != Announcement::noannouncement) || rhs); }

/** @param    lhs   left operand
 ** @param    rhs   right operand
 **
 ** @return   lhs < rhs
 **/
bool
operator<(Announcement const lhs, Announcement const rhs) noexcept
{
  if (lhs == rhs)
    return false;
  return (lhs <= rhs);
} // bool operator<(Announcement lhs, Announcement rhs)

/** @param    lhs   left operand
 ** @param    rhs   right operand
 **
 ** @return   lhs <= rhs
 **/
bool
operator<=(Announcement const lhs, Announcement const rhs) noexcept
{
  switch(lhs) {
  case Announcement::noannouncement:
    return true;
  case Announcement::no120:
    if (rhs == Announcement::no120)
      return true;
  case Announcement::no90:
    if (rhs == Announcement::no90)
      return true;
  case Announcement::no60:
    if (rhs == Announcement::no60)
      return true;
  case Announcement::no30:
    if (rhs == Announcement::no30)
      return true;
  case Announcement::no0:
    if (rhs == Announcement::no0)
      return true;
    return false;
  case Announcement::reply:
  case Announcement::no120_reply:
  case Announcement::no90_reply:
  case Announcement::no60_reply:
  case Announcement::no30_reply:
  case Announcement::no0_reply:
    return false;
  } // switch(announcement)

  return false;
} // bool operator<=(Announcement lhs, Announcement rhs)

/** -> result
 **
 ** @param    announcement   announcement
 **
 ** @return   whether the announcement is a real announcement (no reply)
 **/
bool
is_real(Announcement const announcement) noexcept
{
  switch(announcement) {
  case Announcement::noannouncement:
    return false;
  case Announcement::no120:
  case Announcement::no90:
  case Announcement::no60:
  case Announcement::no30:
  case Announcement::no0:
    return true;
  case Announcement::reply:
  case Announcement::no120_reply:
  case Announcement::no90_reply:
  case Announcement::no60_reply:
  case Announcement::no30_reply:
  case Announcement::no0_reply:
    return false;
  } // switch(announcement)

  DEBUG_ASSERTION(false,
                  "is_real(Announcement):\n"
                  "  announcement not found: " << announcement);
  return false;
} // bool is_real(Announcement announcement)

/** -> result
 **
 ** @param    announcement   announcement
 **
 ** @return   whether the announcement is a reply
 **/
bool
is_reply(Announcement const announcement) noexcept
{
  switch(announcement) {
  case Announcement::noannouncement:
    return false;
  case Announcement::no120:
  case Announcement::no90:
  case Announcement::no60:
  case Announcement::no30:
  case Announcement::no0:
    return false;
  case Announcement::reply:
  case Announcement::no120_reply:
  case Announcement::no90_reply:
  case Announcement::no60_reply:
  case Announcement::no30_reply:
  case Announcement::no0_reply:
    return true;
  } // switch(announcement)

  DEBUG_ASSERTION(false,
                  "is_reply(Announcement):\n"
                  "  announcement not found: " << announcement);
  return false;
} // bool is_reply(Announcement announcement)

/** -> result
 **
 ** @param    announcement   announcement
 **
 ** @return   reply version of 'announcement'
 **/
Announcement
to_reply(Announcement const announcement)
{
  switch(announcement) {
  case Announcement::noannouncement:
    DEBUG_ASSERTION(false,
                    "to_reply(announcement):\n"
                    "  announcement '" << to_string(announcement) << "' not valid.");
    return Announcement::noannouncement;
  case Announcement::no120:
    return Announcement::no120_reply;
  case Announcement::no90:
    return Announcement::no90_reply;
  case Announcement::no60:
    return Announcement::no60_reply;
  case Announcement::no30:
    return Announcement::no30_reply;
  case Announcement::no0:
    return Announcement::no0_reply;
  default:
    DEBUG_ASSERTION(false,
                    "to_reply(announcement):\n"
                    "  announcement '" << to_string(announcement) << "' not valid.");
  } // switch(announcement)

  return Announcement::noannouncement;
} // Announcement to_reply(Announcement announcement)

/** -> result
 **
 ** @param    announcement   announcement
 **
 ** @return   on which announcement this announcement was replied
 **/
Announcement
from_reply(Announcement const announcement)
{
  switch(announcement) {
  case Announcement::noannouncement:
  case Announcement::no120:
  case Announcement::no90:
  case Announcement::no60:
  case Announcement::no30:
  case Announcement::no0:
  case Announcement::reply:
    DEBUG_ASSERTION(false,
                    "from_reply(announcement):\n"
                    "  announcement '" << to_string(announcement) << "' not valid.");
    return Announcement::noannouncement;
  case Announcement::no120_reply:
    return Announcement::no120;
  case Announcement::no90_reply:
    return Announcement::no90;
  case Announcement::no60_reply:
    return Announcement::no60;
  case Announcement::no30_reply:
    return Announcement::no30;
  case Announcement::no0_reply:
    return Announcement::no0;
  } // switch(announcement)

  DEBUG_ASSERTION(false,
                  "is_reply(Announcement):\n"
                  "  announcement not found: " << announcement);
  return Announcement::noannouncement;
} // Announcement from_reply(Announcement announcement)

/** -> result
 **
 ** @param    announcement   announcement
 **
 ** @return   previous announcement
 **/
Announcement
previous(Announcement const announcement)
{
  switch (announcement) {
  case Announcement::no120:
    return Announcement::noannouncement;
  case Announcement::no90:
    return Announcement::no120;
  case Announcement::no60:
    return Announcement::no90;
  case Announcement::no30:
    return Announcement::no60;
  case Announcement::no0:
    return Announcement::no30;
  default:
    DEBUG_ASSERTION(false,
                    "previous(announcement)\n"
                    "  there exists no previous announcement of "
                    << announcement);
  } // switch (announcement)

  return Announcement::noannouncement;
} // Announcement previous(Announcement announcement)

/** -> result
 **
 ** @param    announcement   announcement
 **
 ** @return   next announcement
 **/
Announcement
next(Announcement const announcement)
{
  switch (announcement) {
  case Announcement::noannouncement:
    return Announcement::no120;
  case Announcement::no120:
    return Announcement::no90;
  case Announcement::no90:
    return Announcement::no60;
  case Announcement::no60:
    return Announcement::no30;
  case Announcement::no30:
    return Announcement::no0;
  default:
    DEBUG_ASSERTION(false,
                    "next(announcement)\n"
                    "  there exists no next announcement of "
                    << announcement);
  } // switch (announcement)

  return Announcement::noannouncement;
} // Announcement next(Announcement announcement)

/** -> result
 **
 ** @param    announcement   announcement
 **
 ** @return   the points needed to redeem the announcement
 **/
unsigned
needed_points(Announcement const announcement) noexcept
{
  switch (announcement) {
  case Announcement::noannouncement:
    return 120;
  case Announcement::no120:
    return 240 - 120 + 1;
  case Announcement::no90:
    return 240 -  90 + 1;
  case Announcement::no60:
    return 240 -  60 + 1;
  case Announcement::no30:
    return 240 -  30 + 1;
  case Announcement::no0:
    return 240 -   0 + 1;
  case Announcement::reply:
    return 120;
  case Announcement::no120_reply:
    return 120;
  case Announcement::no90_reply:
    return 90;
  case Announcement::no60_reply:
    return 60;
  case Announcement::no30_reply:
    return 30;
  case Announcement::no0_reply:
    return 0;
  } // switch (announcement)

  return 0;
} // unsigned needed_points(Announcement announcement)

/** -> result
 **
 ** @param    announcement   announcement
 **
 ** @return   the points needed for the opposite team to win
 **/
unsigned
point_limit_for_opposite_team(Announcement const announcement) noexcept
{
  return 240 - needed_points(announcement) + 1;
} // unsigned point_limit_for_opposite_team(Announcement announcement)
