/**********************************************************************
 *
 *   freedoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "fast_play.h"

FastPlay fast_play = {};

FastPlayBit const FastPlay::party_start = FastPlayBit::party_start;
FastPlayBit const FastPlay::player = FastPlayBit::player;
FastPlayBit const FastPlay::random_ai = FastPlayBit::random_ai;
FastPlayBit const FastPlay::pause = FastPlayBit::pause;
FastPlayBit const FastPlay::gameplay_information = FastPlayBit::gameplay_information;
FastPlayBit const FastPlay::game_finished = FastPlayBit::game_finished;
FastPlayBit const FastPlay::party_finished = FastPlayBit::party_finished;
FastPlayBit const FastPlay::show_all_hands = FastPlayBit::show_all_hands;
FastPlayBit const FastPlay::hide_bug_report_window = FastPlayBit::hide_bug_report_window;
FastPlayBit const FastPlay::quit_when_finished = FastPlayBit::quit_when_finished;
FastPlayBit const FastPlay::seed_info = FastPlayBit::seed_info;

/** constructor
 **/
FastPlay::FastPlay() noexcept = default;

/** constructor
 **
 ** @param    v  fast play bit
 **/
FastPlay::FastPlay(FastPlayBit const v) noexcept
: value(static_cast<int>(v))
{ }

/** destructor
 **/
FastPlay::~FastPlay() noexcept = default;

/** conversion into a boolean
 **/

FastPlay::operator bool() noexcept
{
  return value != 0;
}

/** -> result
 **
 ** @param    rhs   bit to check
 **
 ** @return   checks whether rhs is set
 **/
FastPlay
FastPlay::operator&(FastPlayBit rhs) const noexcept
{
  if ((value & static_cast<int>(rhs)) != 0) {
    return rhs;
  } else {
    return {};
  }
}

/** add a bit
 **
 ** @param    rhs   bit to add
 **
 ** @return   set bits
 **/
FastPlay&
FastPlay::operator|=(FastPlayBit rhs) noexcept
{
  value |= static_cast<int>(rhs);
  return *this;
}

/** add a bit
 **
 ** @param    rhs   bit to add
 **/
void
FastPlay::add(FastPlayBit rhs) noexcept
{
  value |= static_cast<int>(rhs);
}

/** remove a bit
 **
 ** @param    rhs   bit to removed
 **/
void
FastPlay::remove(FastPlayBit rhs) noexcept
{
  value &= ~static_cast<int>(rhs);
}

/** merges two bits
 **
 ** @param    lhs   left operand
 ** @param    rhs   right operand
 **
 ** @return   merged bits
 **/
FastPlay
operator|(FastPlayBit const lhs, FastPlayBit const rhs) noexcept
{
  FastPlay res(lhs);
  res.value |= static_cast<int>(rhs);
  return res;
}
