/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

class FastPlay;
extern FastPlay fast_play;

enum class FastPlayBit {
    party_start          = 1,                         //   1 automatical start party
    player               = party_start          << 1, //   2 change human players to ais
    random_ai            = player               << 1, //   4 set the players to the random ai
    pause                = random_ai            << 1, //   8 skip pauses
    gameplay_information = pause                << 1, //  16 skip gameplay information dialogs
    game_finished        = gameplay_information << 1, //  32 skip game finished dialog
    party_finished        = game_finished       << 1, //  64 skip party finished dialog
    show_all_hands      = party_finished        << 1, // 128 show all hands
    hide_bug_report_window     = show_all_hands << 1, // 256 do not show the bug report replay window
    quit_when_finished = hide_bug_report_window << 1, // 512 quit when finished
    seed_info              = quit_when_finished << 1 // 1024 only write the seed on stdout
}; // enum class FastPlayBit

class FastPlay {
  friend FastPlay operator|(FastPlayBit lhs, FastPlayBit rhs) noexcept;
  public:
    static FastPlayBit const party_start;
    static FastPlayBit const player;
    static FastPlayBit const random_ai;
    static FastPlayBit const pause;
    static FastPlayBit const gameplay_information;
    static FastPlayBit const game_finished;
    static FastPlayBit const party_finished;
    static FastPlayBit const show_all_hands;
    static FastPlayBit const hide_bug_report_window;
    static FastPlayBit const quit_when_finished;
    static FastPlayBit const seed_info;

  public:
    FastPlay() noexcept;
    FastPlay(FastPlayBit v) noexcept;

    ~FastPlay() noexcept;

    operator bool() noexcept;

    FastPlay operator&(FastPlayBit rhs) const noexcept;

    FastPlay& operator|=(FastPlayBit rhs) noexcept;

    void add(FastPlayBit rhs) noexcept;
    void remove(FastPlayBit rhs) noexcept;

  private:
    int value = 0;
};

FastPlay operator|(FastPlayBit lhs, FastPlayBit rhs) noexcept;
