/**********************************************************************
 *
 *   freedoko a Doppelkopf-Game
 *
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "game_status.h"

// status of the current game
GameStatus game_status = GameStatus::programstart;

/** -> result
 **
 ** @param    game_status   game status
 **
 ** @return   name of 'game_status'
 **/
string
to_string(GameStatus const game_status) noexcept
{
  switch (game_status) {
  case GameStatus::programstart:
    return "program start";
    (void)_("GameStatus::program start");
  case GameStatus::quit:
    return "quit";
    (void)_("GameStatus::quit");
  case GameStatus::party_new:
    return "party new";
    (void)_("GameStatus::party new");
  case GameStatus::party_initial_loaded:
    return "party initial loaded";
    (void)_("GameStatus::party initial loaded");
  case GameStatus::party_loaded:
    return "party loaded";
    (void)_("GameStatus::party loaded");
  case GameStatus::party_play:
    return "party play";
    (void)_("GameStatus::party play");
  case GameStatus::party_finished:
    return "party finished";
    (void)_("GameStatus::party finished");
  case GameStatus::game_new:
    return "game new";
    (void)_("GameStatus::game new");
  case GameStatus::game_init:
    return "game init";
    (void)_("GameStatus::game init");
  case GameStatus::game_reservation:
    return "game reservation";
    (void)_("GameStatus::game reservation");
  case GameStatus::game_redistribute:
    return "game redistribute";
    (void)_("GameStatus::game redistribute");
  case GameStatus::game_poverty_shift:
    return "game poverty shift";
    (void)_("GameStatus::game poverty shift");
  case GameStatus::game_start:
    return "game start";
    (void)_("GameStatus::game start");
  case GameStatus::game_play:
    return "game play";
    (void)_("GameStatus::game play");
  case GameStatus::game_full_trick:
    return "game full trick";
    (void)_("GameStatus::game full trick");
  case GameStatus::game_trick_taken:
    return "game trick taken";
    (void)_("GameStatus::game trick taken");
  case GameStatus::game_finished:
    return "game finished";
    (void)_("GameStatus::game finished");
  case GameStatus::game_closed:
    return "game closed";
    (void)_("GameStatus::game closed");
  } // switch(game_status)

  DEBUG_ASSERTION(false, "::to_string(GameStatus):\n"
                         "  game status not found: "
                             << static_cast<int>(game_status));

  return "";
} // string to_string(GameStatus game_status)

/** -> result
 **
 ** @param    game_status   game status
 **
 ** @return   translation of the game status
 **/
string
gettext(GameStatus const game_status)
{
  return gettext("GameStatus::" + to_string(game_status));
}

/** writes the gamestatus into the stream
 **
 ** @param    ostr          output stream
 ** @param    game_status   game status
 **
 ** @return   output stream
 **/
ostream &
operator<<(ostream &ostr, GameStatus const game_status)
{
  return (ostr << ::to_string(game_status));
}

/** -> result
 **
 ** @param    lhs   left operand
 ** @param    rhs   left operand, must be party or game
 **
 ** @return   whether the gamestatus is in a party or in a game
 **/
bool operator&(GameStatus const lhs, GameStatus const rhs)
{
  if (rhs == GameStatus::party) {
    return (static_cast<int>(lhs) & static_cast<int>(rhs));
  } else if (rhs == GameStatus::game) {
    return (static_cast<int>(lhs) & static_cast<int>(rhs));
  } else {
    DEBUG_ASSERTION(false, "GameStatus & GameStatus: right hand side must be "
                           "party or game but is " +
                               to_string(rhs));
  }
  return false;
} // bool operator&(GameStatus lhs, GameStatus rhs)

/** @result   whether the status is in a running game
 **/
bool
in_running_game() noexcept
{
  switch (::game_status) {
  case GameStatus::game_play:
  case GameStatus::game_full_trick:
  case GameStatus::game_trick_taken:
    return true;
  default:
    return false;
  }
}
