/**********************************************************************
 *
 *   freedoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "marriage_selector.h"

/** -> result
 **
 ** @param    marriage_selector   marriage selector
 **
 ** @return   name of 'marriage_selector'
 **/
string
to_string(MarriageSelector const marriage_selector) noexcept
{
  switch(marriage_selector) {
  case MarriageSelector::silent:
    return "silent";
    (void)_("MarriageSelector::silent");
  case MarriageSelector::team_set:
    return "team set";
    (void)_("MarriageSelector::team set");
  case MarriageSelector::first_foreign:
    return "first foreign";
    (void)_("MarriageSelector::first foreign");
  case MarriageSelector::first_trump:
    return "first trump";
    (void)_("MarriageSelector::first trump");
  case MarriageSelector::first_color:
    return "first color";
    (void)_("MarriageSelector::first color");
  case MarriageSelector::first_club:
    return "first club";
    (void)_("MarriageSelector::first club");
  case MarriageSelector::first_spade:
    return "first spade";
    (void)_("MarriageSelector::first spade");
  case MarriageSelector::first_heart:
    return "first heart";
    (void)_("MarriageSelector::first heart");
  } // switch(marriage_selector)

  DEBUG_ASSERTION(false,
                  "to_string(MarriageSelector):\n"
                  "  no name found for " << static_cast<int>(marriage_selector)
                 );

  return "";
} // string to_string(MarriageSelector marriage_selector)

/** -> result
 **
 ** @param    marriage_selector
 **
 ** @return   translation of the marriage selector
 **/
string
gettext(MarriageSelector const marriage_selector)
{
  return gettext("MarriageSelector::" + to_string(marriage_selector));
}

/** writes the marriage selector into the stream
 **
 ** @param    ostr                output stream
 ** @param    marriage_selector   marriage selector
 **
 ** @return   output stream
 **/
ostream&
operator<<(ostream& ostr, MarriageSelector const marriage_selector)
{ return (ostr << to_string(marriage_selector)); }

/** -> result
 **
 ** @param    name   marriage selector name
 **
 ** @return   the marriage selector with name 'name'
 **/
MarriageSelector
marriage_selector_from_string(string const& name)
{
  for (auto const ms : marriage_selector_list)
    if (name == to_string(ms))
      return ms;

  throw ReadException("unknown marriage selector '" + name + "'");
  return MarriageSelector::team_set;
} // MarriageSelector marriage_selector_from_string(string name)

/** @return   whether the marriage selector is not set
 **/
bool
operator!(MarriageSelector const marriage_selector) noexcept
{ return (marriage_selector == MarriageSelector::team_set); }
