/**********************************************************************
 *
 *   freedoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"
#include "rotation.h"

/** -> result
 **
 ** @param    rotation   rotation
 **
 ** @return   'rotation' clockwise rotated
 **/
Rotation
clockwise(Rotation const rotation) noexcept
{
  switch (rotation) {
  case Rotation::up:
    return Rotation::left;
  case Rotation::down:
    return Rotation::right;
  case Rotation::right:
    return Rotation::up;
  case Rotation::left:
    return Rotation::down;
  } // switch(rotation)

  DEBUG_ASSERTION(false,
                  "ROTATION::conterclockwise(rotation):\n"
                  "  behind 'switch(rotation)'");

  return Rotation::up;
} // Rotation clockwise(Rotation rotation)

/** -> result
 **
 ** @param    rotation   rotation
 **
 ** @return   'rotation' counterclockwise rotated
 **/
Rotation
counterclockwise(Rotation const rotation) noexcept
{
  switch (rotation) {
  case Rotation::up:
    return Rotation::right;
  case Rotation::down:
    return Rotation::left;
  case Rotation::right:
    return Rotation::down;
  case Rotation::left:
    return Rotation::up;
  } // switch(rotation)

  DEBUG_ASSERTION(false,
                  "ROTATION::conterclockwise(rotation):\n"
                  "  behind 'switch(rotation)'");

  return Rotation::up;
} // Rotation counterclockwise(Rotation rotation)

/** Conversion into a string
 **
 ** @param    rotation   rotation
 **
 ** @return   string of the rotation
 **/
string
to_string(Rotation const rotation) noexcept
{
  switch (rotation) {
  case Rotation::up:
    return "up";
  case Rotation::down:
    return "down";
  case Rotation::right:
    return "right";
  case Rotation::left:
    return "left";
  } // switch(rotation)
  return {};
} // string to_string(Rotation rotation)

/** write the rotation into the output stream
 **
 ** @param    ostr       output stream
 ** @param    rotation   rotation 
 **
 ** @return   output stream
 **/
ostream&
operator<<(ostream& ostr, Rotation const rotation)
{
  ostr << to_string(rotation);
  return ostr;
} // ostream& operator<<(ostream& ostr, Rotation rotation)
