/**********************************************************************
 *
 *   freedoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "team.h"

/** -> result
 **
 ** @param    team   team
 **
 ** @return   name of the team
 **/
string
to_string(Team const team) noexcept
{
  switch(team) {
  case Team::re:
    return "re";
    (void)_("Team::re");
  case Team::contra:
    return "contra";
    (void)_("Team::contra");
  case Team::maybe_re:
    return "maybe re";
    (void)_("Team::maybe re");
  case Team::maybe_contra:
    return "maybe contra";
    (void)_("Team::maybe contra");
  case Team::unknown:
    return "unknown team";
    (void)_("Team::unknown team");
  case Team::noteam:
    return "no team";
    (void)_("Team::no team");
  } // switch(team)

  DEBUG_ASSERTION(false,
                  "to_string(Team):\n"
                  "  team not found: " << static_cast<int>(team));
  return {};
} // string to_string(Team team)

/** -> result
 **
 ** @param    team   team
 **
 ** @return   translation of the team name
 **/
string
gettext(Team const team)
{
  return gettext("Team::" + to_string(team));
}

/** writes the team into the stream
 **
 ** @param    ostr   output stream
 ** @param    team   team
 **
 ** @return   output stream
 **/
ostream&
operator<<(ostream& ostr, Team const team)
{ return (ostr << to_string(team)); }

/** -> result
 **
 ** @param    team   team
 **
 ** @return   name of the team
 **/
string
short_string(Team const team) noexcept
{
  switch(team) {
  case Team::re:
    return "re";
    (void)_("Team::re");
  case Team::contra:
    return "contra";
    (void)_("Team::contra");
  case Team::maybe_re:
    return "?re?";
    (void)_("Team::?re?");
  case Team::maybe_contra:
    return "?contra?";
    (void)_("Team::?contra?");
  case Team::unknown:
    return "?";
    (void)_("Team::?");
  case Team::noteam:
    return "-";
    (void)_("Team::-");
  } // switch(team)

  DEBUG_ASSERTION(false,
                  "short_string(Team):\n"
                  "  team not found: " << static_cast<int>(team));

  return {};
} // string short_string(Team team)

/** -> result
 **
 ** @param    name   team name
 **
 ** @return   the team with name 'name'
 **/
Team
team_from_string(string const& name)
{
  for (auto const t : team_list)
    if (name == to_string(t))
      return t;

  throw ReadException("unknown team '" + name + "'");
  return Team::noteam;
} // Team from_string(string name)

/** converts a maybe team to a real team
 **
 ** @param    team   team to convert
 **
 ** @result   re for 'team == re' or 'team == maybe_re',
 **           contra for 'team == contra' or 'team == maybe_contra'
 **           unknown otherwise
 **/
Team
maybe_to_team(Team const team) noexcept
{
  switch(team) {
  case Team::re:
  case Team::maybe_re:
    return Team::re;
  case Team::contra:
  case Team::maybe_contra:
    return Team::contra;
  default:
    return Team::unknown;
  } // switch(team)
} // Team maybe_to_team(Team team)

/** converts a real team to a maybe team
 **
 ** @param    team   team to convert
 **
 ** @result   maybe_re for 'team == re' or 'team == maybe_re',
 **           maybe_contra for 'team == contra' or 'team == maybe_contra'
 **           unknown otherwise
 **/
Team
team_to_maybe(Team const team) noexcept
{
  switch(team) {
  case Team::re:
  case Team::maybe_re:
    return Team::maybe_re;
  case Team::contra:
  case Team::maybe_contra:
    return Team::maybe_contra;
  default:
    return Team::unknown;
  } // switch(team)
} // Team team_to_maybe(Team team)

/** -> result
 **
 ** @param    team   team to flip
 **
 ** @result   contra for 'team == re',
 **           maybe_contra for 'team == maybe_re',
 **           re for 'team == contra'
 **           maybe_re for 'team == maybe_contra'
 **           unknown otherwise
 **/
Team
opposite(Team const team) noexcept
{
  switch(team) {
  case Team::re:
    return Team::contra;
  case Team::maybe_re:
    return Team::maybe_contra;
  case Team::contra:
    return Team::re;
  case Team::maybe_contra:
    return Team::maybe_re;
  default:
    return Team::unknown;
  } // switch(team)
} // Team opposite(Team team)

/** -> result
 **
 ** @param    team   team to flip
 **
 ** @result   whether the team is real (that means 're' or 'contra')
 **/
bool
is_real(Team const team) noexcept
{
  return (   (team == Team::re)
          || (team == Team::contra));
} // bool is_real(Team team)
