/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include "../basetypes/announcement.h"
#include "../basetypes/team.h"

class Game;
class Player;

/**
 ** The announcements of a game
 **/
class Announcements {
  friend class Game;
  public:
  explicit Announcements(Game& game);
  Announcements(Announcements const& announcements, Game& game);

  Announcements() = delete;
  Announcements(Announcements const&) = delete;
  Announcements& operator=(Announcements const&) = delete;

  AnnouncementWithTrickno last(Team team) const;
  AnnouncementWithTrickno last(Player const& player) const;
  vector<AnnouncementWithTrickno> const& all(Player const& player) const;

  void init();
  void request_from_players();
  void evaluate_trick(Trick const& trick);

  void make_announcement(Announcement announcement,
                         Player const& player);
  bool is_valid(Announcement announcement,
                Player const& player) const;
  Announcement valid_announcement(Player const& player) const;
  bool last_chance_to_announce(Announcement announcement,
                               Player const& player) const;

  void remove_all_but_re_contra();

  Signal<void(Announcement, Player const&)>& signal_announcement_made();

  private:
  Game* const game_;
  vector<vector<AnnouncementWithTrickno> > announcements_;
  Signal<void(Announcement, Player const&)> signal_announcement_made_;
}; // class Announcements
