/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include "../card/card.h"
class Game;

/**
 ** The tricks of a game
 **/
class GameCards {
  friend class Game;
  public:
  explicit GameCards(Game& game);
  GameCards(GameCards const& cards, Game& game);

  GameCards() = delete;
  GameCards(GameCards const&) = delete;
  GameCards& operator=(GameCards const&) = delete;

  void update();
  void update_swines();
  void update_hyperswines();

  unsigned count() const;
  unsigned count_unique() const;
  unsigned count(Card::TColor tcolor) const;
  unsigned count_unique(Card::TColor tcolor) const;
  unsigned count_trumps() const;
  unsigned count_played_trumps() const;
  unsigned count_played() const;
  unsigned count_remaining() const;
  unsigned count_played(Card card) const;
  Card::Color trumpcolor() const;
  static Card::Color trumpcolor(GameType gametype);
  vector<Card::Color> colors() const;
  vector<Card> const& cards(Card::TColor tcolor) const;
  vector<Card> const& trumps() const;

  bool less(Card lhs, Card rhs) const;
  Card next_lower_card(Card card) const;
  Card next_higher_card(Card card) const;
  Card next_higher_card(Card card, unsigned n) const;
  vector<Card> lower_cards(Card card) const;
  vector<Card> higher_cards(Card card) const;
  vector<Card> cards_between(Card lhs, Card rhs) const;
  vector<Card> lower_trumps(Card card) const;

  bool is_special(HandCard const& card) const;
  Card fox() const;
  Card dolle() const;
  Card swine() const;
  Card hyperswine() const;

  private:
  Game* const game_;
  // the cards in the colors (in ascending order)
  vector<vector<Card>> cards_;
}; // class GameCards
