/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include "../basetypes/game_type.h"
#include "gameplay_action.h"

class Card;
class Hand;
class Game;
class Player;

// The gameplay, saved as actions
class Gameplay {
  friend class Game;
  public:
  Gameplay() = delete;
  explicit Gameplay(Game const& game);
  Gameplay(Game const& game, Gameplay const& gameplay);
  Gameplay(Gameplay const&) = delete;
  Gameplay& operator=(Gameplay const&) = delete;
  ~Gameplay();

  unique_ptr<Gameplay> clone(Game const& game) const;

  ostream& write(ostream& ostr) const;

  void cards_distributed();
  void redistribute();
  void game_start();


  vector<unique_ptr<GameplayAction>> const& actions() const;
  void gameplay_action(GameplayAction const& action);

  private:
  Game const* const game_ = nullptr;
  unsigned seed_ = UINT_MAX;
  GameType gametype_ = GameType::normal;
  unsigned soloplayer_ = UINT_MAX;
  vector<Hand> hands_;
  vector<unique_ptr<GameplayAction>> actions_;
}; // class Gameplay

ostream& operator<<(ostream& ostr, Gameplay const& gameplay);
