/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#include "marriage.h"

#include "../../utils/string.h"

namespace GameplayActions {

/** constructor
 **
 ** @param    bridegroom   player with the marriage
 ** @param    bride   the partner
 **/
Marriage::Marriage(unsigned const bridegroom,
                   unsigned const bride) :
  BasePlayer(bridegroom),
  bride(bride)
{ }

/** constructor
 **
 ** @param    line   line with the data
 **
 ** @todo   old format
 **/
Marriage::Marriage(string line) :
  BasePlayer()
{
#ifndef OUTDATED
  // old format in the bug report:
  //   Played: Player 2: club ace
  if (string(line, 0, strlen("Marriage: ")) == "Marriage: ") {
    // ToDo

    return ;
  }
#endif

  // ex: marriage = 0, 1
  auto const pre = to_string(this->type()) + " =";
  if (string(line, 0, pre.length()) != pre)
    return ;

  istringstream istr(string(line, pre.length()));
  istr >> this->player;
  if (istr.get() != ',')
    return ;
  istr >> this->bride;
} // Marriage(string line)

/** -> result
 **
 ** @param    action   action to compare with
 **
 ** @return   whether the action is equal to this
 **/
bool
Marriage::equal(GameplayAction const& action) const
{
  return (   this->BasePlayer::equal(action)
          && (this->bride == dynamic_cast<Marriage const&>(action).bride));
} // bool Marriage::equal(GameplayAction action) const

/** writes the action in the output stream
 **
 ** @param    ostr   output stream
 **/
void
Marriage::write(ostream& ostr) const
{
  ostr << this->type();
  ostr << " = " << this->player << ", " << this->bride;
} // void write(ostream& ostr) const

/** @return   a translation representing the data
 **/
string
Marriage::data_translation() const
{
  return std::to_string(this->bride);
} // virtual string Marriage::data_translation() const

} // namespace GameplayActions
