/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include "../player/player.h"
class Game;
using GamePlayer = Player;
class Human;
#include "players_iterator.h"

#include "../party/players.h"

/**
 ** The players of a game
 **/
class GamePlayers {
  public:
    using Player = GamePlayer;
    using iterator = GamePlayersIterator;
    using const_iterator = GamePlayersConstIterator;

    explicit GamePlayers(Game& game);
    GamePlayers(Game& game, GamePlayers const& old_players, vector<unique_ptr<PartyPlayer>>& new_players);
    GamePlayers(Game& game, GamePlayers const& old_players, vector<Player*>& new_players);

    GamePlayers() = delete;
    GamePlayers(GamePlayers const&) = delete;
    GamePlayers& operator=(GamePlayers const&) = delete;

    void self_check() const;
    void reset() noexcept;

    bool empty() const noexcept;
    unsigned size() const noexcept;

    iterator begin() noexcept;
    iterator end() noexcept;
    const_iterator begin() const noexcept;
    const_iterator end() const noexcept;

    Player& player(unsigned p);
    Player const& player(unsigned p) const;

    unsigned no(Player const& player) const noexcept;
    bool contains(Player const& player) const noexcept;

    Player const& current_player() const;
    Player& current_player();
    void set_current_player(Player const& player);
    void next_current_player() noexcept;

    bool exists_soloplayer() const noexcept;
    bool is_soloplayer(Player const& player) const noexcept;
    Player const& soloplayer() const;
    Player& soloplayer();
    void set_soloplayer(Player& player);
    void unset_soloplayer() noexcept;

    Player const& poverty_partner() const;

    unsigned count_humans() const noexcept;
    Human& human();
    Human const& human() const;

    Player const& following(Player const& player) const;
    Player& following(Player& player);
    Player const& previous(Player const& player) const;

    void sort_hands();
    void mix_hands();

    void write(ostream& ostr) const;
    void write_short(ostream& ostr) const;

    bool load();
    void save() const;

  private:
    Game* const game_;
    vector<GamePlayer*> players_;
    Player* current_player_ = nullptr;
    Player* soloplayer_ = nullptr;
}; // class GamePlayers
