/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include "../card/hand_cards.h"
class Game;
class Player;

/**
 ** The poverty part of a game
 **/
class GamePoverty {
  friend class Game;
  public:
  explicit GamePoverty(Game& game) noexcept;
  GamePoverty(GamePoverty const& poverty, Game& game);

  GamePoverty(GamePoverty const&) = delete;
  GamePoverty& operator=(GamePoverty const&) = delete;

  bool shifted() const noexcept;
  unsigned shifted_cardno() const noexcept;
  unsigned returned_cardno() const noexcept;
  unsigned returned_trumpno() const noexcept;
  HandCards const& shifted_cards() const;
  HandCards const& returned_cards() const;

  void reset() noexcept;
  void shift();

  Signal<void(Player const&)>& signal_shifting() noexcept;
  Signal<void(Player const&, unsigned)>& signal_shift() noexcept;
  Signal<void(Player const&, unsigned)>& signal_ask() noexcept;
  Signal<void(Player const&)>& signal_take_denied() noexcept;
  Signal<void()>& signal_take_denied_by_all() noexcept;
  Signal<void(Player const&, unsigned, unsigned)>& signal_take_accepted() noexcept;
  Signal<void(Player const&)>& signal_get_cards_to_shift() noexcept;
  Signal<void(Player const&, unsigned)>& signal_get_take_accept() noexcept;
  Signal<void(Player const&, HandCards)>& signal_get_exchanged_cards() noexcept;
  Signal<void(Player const&, HandCards)>& signal_cards_get_back() noexcept;

  private:
  Game* const game_;
  HandCards shifted_cards_;
  HandCards returned_cards_;
  bool shifted_ = false;

  Signal<void(Player const&)> signal_shifting_;
  Signal<void(Player const&, unsigned)> signal_shift_;
  Signal<void(Player const&, unsigned)> signal_ask_;
  Signal<void(Player const&)> signal_take_denied_;
  Signal<void()> signal_take_denied_by_all_;
  Signal<void(Player const&, unsigned, unsigned)> signal_take_accepted_;
  Signal<void(Player const&)> signal_get_cards_to_shift_;
  Signal<void(Player const&, unsigned)> signal_get_take_accept_;
  Signal<void(Player const&, HandCards)> signal_get_exchanged_cards_;
  Signal<void(Player const&, HandCards)> signal_cards_get_back_;
}; // class GamePoverty
