/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#include "reservation.h"

#include "game.h"

#include "../utils/string.h"

/** constructor
 ** the parts of the reservation are split by ',' in 'line'
 **
 ** @param    line   line to read the reservation from
 **/
Reservation::Reservation(string const& line)
{ 
  // syntax:
  //   'game type', 'marriage selector', swines, hyperswines

  // first split the line in the single parts
  vector<string> const parts(String::split(line, ','));
  auto p = parts.begin();

  this->game_type = game_type_from_string(*p);
  ++p;

  if (this->game_type == GameType::marriage) {
    this->marriage_selector = marriage_selector_from_string(*p);
    ++p;
  } // if (this->game_type == GameType::marriage)

  for ( ; p != parts.end(); ++p) {
    if ((*p) == "offer duty solo") {
      this->offer_duty_solo = true;
    } else if ((*p) == "swines") {
      this->swines = true;
    } else if ((*p) == "hyperswines"){ 
      this->swines = true;
    } else {
      DEBUG_ASSERTION(false,
		      "Reservation(line)\n"
		      "  unknown part '" << *p << "'");
    }
  } // for (p \in parts)
} // Reservation::Reservation(string line)

/** -> result
 **
 ** @param    lhs   first reservation
 ** @param    rhs   second reservation
 **
 ** @return   whether the reservations are equal
 **/
bool
operator==(Reservation const lhs, Reservation const rhs) noexcept
{
  return (   (lhs.game_type == rhs.game_type)
	  && (lhs.marriage_selector == rhs.marriage_selector)
	  && (lhs.offer_duty_solo == rhs.offer_duty_solo)
	  && (lhs.swines == rhs.swines)
	  && (lhs.hyperswines == rhs.hyperswines));
} // bool operator==(Reservation lhs, Reservation rhs)

/** write the reservation in the stream
 **
 ** @param    ostr          output stream to write into
 ** @param    reservation   reservation to write
 **
 ** @return   the output stream
 **/
ostream&
operator<<(ostream& ostr, Reservation const& reservation)
{
  ostr << "game type: " << reservation.game_type << '\n';
  if (reservation.game_type == GameType::marriage)
    ostr << "marriage selector: " << reservation.marriage_selector << '\n';
  if (reservation.offer_duty_solo)
    ostr << "offer duty solo\n";
  if (reservation.swines)
    ostr << "swines\n";
  if (reservation.hyperswines)
    ostr << "hyperswines\n";
  return ostr;
} // ostream& operator<<(ostream& ostr, Reservation reservation)
