/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#include "specialpoint.h"

#include "../party/party.h"
#include "../party/rule.h"
#include "../game/game.h"
#include "../class/readconfig/readconfig.h"

#include "../utils/string.h"

constexpr Specialpoint::Type Specialpoint::types[];

/** -> return
 **
 ** @param    sp   the specialpoint type
 **
 ** @return   whether 'sp' was got by winning
 **/
bool
is_winning(Specialpoint::Type const sp) noexcept
{
  switch (sp) {
  case Specialpoint::Type::won:
  case Specialpoint::Type::no90:
  case Specialpoint::Type::no60:
  case Specialpoint::Type::no30:
  case Specialpoint::Type::no0:
  case Specialpoint::Type::contra_won:
  case Specialpoint::Type::no90_said_120_got:
  case Specialpoint::Type::no60_said_90_got:
  case Specialpoint::Type::no30_said_60_got:
  case Specialpoint::Type::no0_said_30_got:
  case Specialpoint::Type::solo:
    return true;
    break;
  default:
    return false;
    break;
  }; // switch(sp)
} // bool is_winning(Specialpoint::Type sp)

/** -> return
 **
 ** @param    sp   the specialpoint type
 **
 ** @return   whether 'sp' was got by announcement
 **/
bool
is_announcement(Specialpoint::Type const sp) noexcept
{
  switch (sp) {
  case Specialpoint::Type::no120_said:
  case Specialpoint::Type::no90_said:
  case Specialpoint::Type::no60_said:
  case Specialpoint::Type::no30_said:
  case Specialpoint::Type::no0_said:
  case Specialpoint::Type::no120_reply:
  case Specialpoint::Type::no90_reply:
  case Specialpoint::Type::no60_reply:
  case Specialpoint::Type::no30_reply:
  case Specialpoint::Type::no0_reply:
    return true;
    break;
  default:
    return false;
    break;
  }; // switch(sp)
} // bool is_announcement(Specialpoint::Type sp)


/** -> result
 **
 ** @param    sp   special point
 **
 ** @result   points for this specialpoint
 **/
int
value(Specialpoint::Type const sp) noexcept
{
  switch (sp) {
  case Specialpoint::Type::nospecialpoint:
    return 0;
  case Specialpoint::Type::caught_fox:
  case Specialpoint::Type::caught_fox_last_trick:
  case Specialpoint::Type::fox_last_trick:
  case Specialpoint::Type::charlie:
  case Specialpoint::Type::caught_charlie:
  case Specialpoint::Type::dolle_caught_dolle:
  case Specialpoint::Type::heart_trick:
  case Specialpoint::Type::doppelkopf:
  case Specialpoint::Type::won:
  case Specialpoint::Type::no90:
  case Specialpoint::Type::no60:
  case Specialpoint::Type::no30:
  case Specialpoint::Type::no0:
    return 1;
  case Specialpoint::Type::no120_said:
    return 2;
  case Specialpoint::Type::no90_said:
  case Specialpoint::Type::no60_said:
  case Specialpoint::Type::no30_said:
  case Specialpoint::Type::no0_said:
  case Specialpoint::Type::no90_said_120_got:
  case Specialpoint::Type::no60_said_90_got:
  case Specialpoint::Type::no30_said_60_got:
  case Specialpoint::Type::no0_said_30_got:
  case Specialpoint::Type::no120_reply:
  case Specialpoint::Type::no90_reply:
  case Specialpoint::Type::no60_reply:
  case Specialpoint::Type::no30_reply:
  case Specialpoint::Type::no0_reply:
  case Specialpoint::Type::contra_won:
    return 1;
  case Specialpoint::Type::solo:
    return 3;
  case Specialpoint::Type::bock:
    return 0;
  } // switch(sp)

  return 0;
} // int value(Specialpoint::Type sp)

/** -> result
 **
 ** @param    type   the type of the special point
 **
 ** @return   name of the specialpoints
 **/
string
to_string(Specialpoint::Type const type) noexcept
{
  switch(type) {
  case Specialpoint::Type::nospecialpoint:
    return "no specialpoint";
    (void)_("Specialpoint::no specialpoint");
  case Specialpoint::Type::caught_fox:
    return "caught fox";
    (void)_("Specialpoint::caught fox");
  case Specialpoint::Type::caught_fox_last_trick:
    return "caught fox last trick";
    (void)_("Specialpoint::caught fox last trick");
  case Specialpoint::Type::fox_last_trick:
    return "fox last trick";
    (void)_("Specialpoint::fox last trick");
  case Specialpoint::Type::charlie:
    return "charlie";
    (void)_("Specialpoint::charlie");
  case Specialpoint::Type::caught_charlie:
    return "caught charlie";
    (void)_("Specialpoint::caught charlie");
  case Specialpoint::Type::dolle_caught_dolle:
    return "dolle caught dolle";
    (void)_("Specialpoint::dolle caught dolle");
  case Specialpoint::Type::heart_trick:
    return "heart trick";
    (void)_("Specialpoint::heart trick");
  case Specialpoint::Type::doppelkopf:
    return "Doppelkopf";
    (void)_("Specialpoint::Doppelkopf");
  case Specialpoint::Type::won:
    return "won";
    (void)_("Specialpoint::won");
  case Specialpoint::Type::no90:
    return "no 90";
    (void)_("Specialpoint::no 90");
  case Specialpoint::Type::no60:
    return "no 60";
    (void)_("Specialpoint::no 60");
  case Specialpoint::Type::no30:
    return "no 30";
    (void)_("Specialpoint::no 30");
  case Specialpoint::Type::no0:
    return "no 0";
    (void)_("Specialpoint::no 0");
  case Specialpoint::Type::no120_said:
    return "no 120 said";
    (void)_("Specialpoint::no 120 said");
  case Specialpoint::Type::no90_said:
    return "no 90 said";
    (void)_("Specialpoint::no 90 said");
  case Specialpoint::Type::no60_said:
    return "no 60 said";
    (void)_("Specialpoint::no 60 said");
  case Specialpoint::Type::no30_said:
    return "no 30 said";
    (void)_("Specialpoint::no 30 said");
  case Specialpoint::Type::no0_said:
    return "no 0 said";
    (void)_("Specialpoint::no 0 said");
  case Specialpoint::Type::no90_said_120_got:
    return "got 120 against no 90";
    (void)_("Specialpoint::got 120 against no 90");
  case Specialpoint::Type::no60_said_90_got:
    return "got 90 against no 60";
    (void)_("Specialpoint::got 90 against no 60");
  case Specialpoint::Type::no30_said_60_got:
    return "got 60 against no 30";
    (void)_("Specialpoint::got 60 against no 30");
  case Specialpoint::Type::no0_said_30_got:
    return "got 30 against no 0";
    (void)_("Specialpoint::got 30 against no 0");
  case Specialpoint::Type::no120_reply:
    return "no 120 reply said";
    (void)_("Specialpoint::no 120 reply said");
  case Specialpoint::Type::no90_reply:
    return "no 90 reply said";
    (void)_("Specialpoint::no 90 reply said");
  case Specialpoint::Type::no60_reply:
    return "no 60 reply said";
    (void)_("Specialpoint::no 60 reply said");
  case Specialpoint::Type::no30_reply:
    return "no 30 reply said";
    (void)_("Specialpoint::no 30 reply said");
  case Specialpoint::Type::no0_reply:
    return "no 0 reply said";
    (void)_("Specialpoint::no 0 reply said");
  case Specialpoint::Type::contra_won:
    return "contra won";
    (void)_("Specialpoint::contra won");
  case Specialpoint::Type::solo:
    return "solo";
    (void)_("Specialpoint::solo");
  case Specialpoint::Type::bock:
    return "bock";
    (void)_("Specialpoint::bock");
  } // switch(specialpoints)

  return "";
} // string to_string(Specialpoint::Type specialpoint)

/** -> result
 **
 ** @param    name   special point name
 **
 ** @return   the special point with name 'name'
 **/
Specialpoint::Type
specialpoint_type_from_string(string const& name)
{
#ifndef OUTDATED
  // 0.7.5: no 120 -> won
  if (name == "no 120")
    return Specialpoint::Type::won;
  if (name == "no 90 won")
    return Specialpoint::Type::no90;
  if (name == "no 60 won")
    return Specialpoint::Type::no60;
  if (name == "no 30 won")
    return Specialpoint::Type::no30;
  if (name == "no 0 won")
    return Specialpoint::Type::no0;
#endif
  for (auto t : Specialpoint::types)
    if (name == to_string(t))
      return t;

  throw ReadException("unknown special point '" + name + "'");
  return Specialpoint::Type::nospecialpoint;
} // Specialpoint::Type specialpoint_type_from_string(string name)

/** writes the specialpoint type in the output stream
 **
 ** @param    ostr   output stream
 ** @param    type   type to write
 **
 ** @return   output stream
 **/
ostream&
operator<<(ostream& ostr, Specialpoint::Type const type)
{
  return (ostr << to_string(type));
} // ostream operator<<(ostream& ostr, Specialpoint::Type type);

/** constructor
 **
 ** @param    type   the special point type
 ** @param    team   the team which has got the special point
 **/
Specialpoint::Specialpoint(Specialpoint::Type const sp,
                             Team const team) noexcept :
  type(sp),
  team(team),
  counting_team(team),
  value(::value(sp))
{ }

/** constructor
 **
 ** @param    type            the special point type
 ** @param    team            the team which has got the special point
 ** @param    counting_team   the counting team
 **/
Specialpoint::Specialpoint(Specialpoint::Type const type,
                             Team const team,
                             Team const counting_team) noexcept :
  type(type),
  team(team),
  counting_team(counting_team),
  value(::value(type))
{ }

/** constructor
 **
 ** @param    type    the special point type
 ** @param    value   the value of the special point (used for bock rounds)
 **/
Specialpoint::Specialpoint(Specialpoint::Type const type,
                             int const value) noexcept :
  type(type),
  value(value)
{ }

/** Constructor
 **
 ** @param    istr   input stream to read the special points from
 **/
Specialpoint::Specialpoint(istream& istr)
{
  Config config;
  istr >> config;

  this->type = specialpoint_type_from_string(config.name);
  this->value = ::value(this->type);
  string::size_type pos_begin = 0;
  while (   (pos_begin < config.value.size())
         && isspace(config.value[pos_begin]))
    ++pos_begin;
  string::size_type pos_end = config.value.find(',');
  if (pos_end == string::npos) {
    return ;
  }
  this->team = team_from_string(string(config.value,
                                       pos_begin, pos_end - pos_begin));
  pos_begin = pos_end + 1;
  while (   (pos_begin < config.value.size())
         && isspace(config.value[pos_begin]))
    ++pos_begin;
  pos_end = config.value.find(',', pos_begin);
  if (pos_end == string::npos) {
    return ;
  }
  this->counting_team = team_from_string(string(config.value,
                                               pos_begin, pos_end - pos_begin));
  pos_begin = pos_end + 1;
  while (   (pos_begin < config.value.size())
         && isspace(config.value[pos_begin]))
    ++pos_begin;
  pos_end = config.value.find(',', pos_begin);
  if (pos_end == string::npos) {
    return ;
  }
  this->player_get_no = Unsigned(string(config.value,
                                        pos_begin, pos_end - pos_begin)
                                );
  pos_begin = pos_end + 1;
  while (   (pos_begin < config.value.size())
         && isspace(config.value[pos_begin]))
    ++pos_begin;
  pos_end = config.value.find(',', pos_begin);
  if (pos_end == string::npos) {
    return ;
  }
  this->player_of_no = Unsigned(string(config.value,
                                       pos_begin, pos_end - pos_begin)
                               );
  pos_begin = pos_end + 1;
  while (   (pos_begin < config.value.size())
         && isspace(config.value[pos_begin]))
    ++pos_begin;
  pos_end = config.value.size();
  if (pos_end == string::npos) {
    return ;
  }

  String::from_string(string(config.value,
                                        pos_begin, pos_end - pos_begin),
                                 this->value);
  DEBUG_ASSERTION((this->type == Specialpoint::Type::bock)
                  || (this->value == ::value(this->type)),
                  "Specialpoints::Specialpoints(istream& istr)\n"
                  "  type = " << this->type << " != bock and "
                  "value = " << this->value << " != "
                  << ::value(this->type)
                  << " = value of the type");
} // Specialpoint::Specialpoint(istream& istr)

/** -> result
 **
 ** @param    team   teamvector
 **
 ** @return   whether the specialpoint is valid at the end of the game
 **/
bool
Specialpoint::is_valid(vector<Team> const& team) const
{
  switch(type) {
  case Type::caught_fox:
  case Type::caught_fox_last_trick:
  case Type::caught_charlie:
  case Type::dolle_caught_dolle:
    return (team[this->player_of_no] != team[this->player_get_no]);
  case Type::fox_last_trick:
  case Type::charlie:
  case Type::heart_trick:
  case Type::doppelkopf:
    return true;
  case Type::nospecialpoint:
  case Type::won:
  case Type::no90:
  case Type::no60:
  case Type::no30:
  case Type::no0:
  case Type::no120_said:
  case Type::no90_said:
  case Type::no60_said:
  case Type::no30_said:
  case Type::no0_said:
  case Type::no90_said_120_got:
  case Type::no60_said_90_got:
  case Type::no30_said_60_got:
  case Type::no0_said_30_got:
  case Type::no120_reply:
  case Type::no90_reply:
  case Type::no60_reply:
  case Type::no30_reply:
  case Type::no0_reply:
  case Type::contra_won:
  case Type::solo:
  case Type::bock:
    return true;
  } // switch(specialpoints)
  return false;
} // bool Specialpoint::is_valid(vector<Team> team) const

/** writes the specialpoint class in the output stream
 **
 ** @param    ostr   output stream
 ** @param    sp     specialpoint to write
 **
 ** @return   output stream
 **/
ostream&
operator<<(ostream& ostr, Specialpoint const& sp)
{
  ostr << sp.type << " = "
    << sp.team << ", "
    << sp.counting_team << ", "
    << Unsigned(sp.player_get_no) << ", "
    << Unsigned(sp.player_of_no) << ", "
    << sp.value
    << '\n';

  return ostr;
} // ostream operator<<(ostream& ostr, Specialpoint sp);

/** writes the specialpoint vector in the output stream
 **
 ** @param    ostr  output stream
 ** @param    spv   specialpoints to write
 **
 ** @return   output stream
 **/
ostream&
operator<<(ostream& ostr, Specialpoints const& spv)
{
  for (auto const sp : spv)
    ostr << sp;

  return ostr;
} // ostream operator<<(ostream& ostr, Specialpoints spv);

/** -> result
 **
 ** @param    spv      vector with the special points
 ** @param    winner   the winnerteam
 ** @param    spv      vector with the special points
 **
 ** @result    sum of the special points for winnerteam minus all specialpoints of the losing team
 **/
int
sum(Specialpoints const& spv, Team const winner, Game const& game)
{
  int re_played = 0;
  int re_ann = 0;
  int re_trick = 0;
  int contra_played = 0;
  int contra_trick = 0;
  int contra_ann = 0;
  int no120s = 1;
  auto const& rule = game.rule();

  for (auto const& sp : spv) {
#ifdef BENDERS
#ifndef RELEASE
    DEBUG_PRINTING((sp.team!=Team::re) &&
                   (sp.team!=Team::contra),
                   "warning: Specialpoints for undetermined team");

    if (   (sp.team != Team::re)
        && (sp.team != Team::contra)
        && (sp.type != Specialpoint::Type::bock) ) {
      COUT << "Game: \n"
        << "  Current trick no: " << game.tricks().current_no()
        << endl;

      COUT << "Teams: " << endl;
      for (unsigned p = 0; p < game.playerno(); p++)
        COUT << p << ": " << game.team(game.player(p)) << endl;

      COUT << "Specialpoint:\n"
        << "  Type: " << sp.type << '\n'
        << "  Team: " << sp.team << '\n'
        << "  player_get_no: " << sp.player_get_no << '\n'
        << "  player_of_no:  " << sp.player_of_no << '\n'
        << endl;
      DEBUG_ASSERTION((sp.team==Team::re)
                      || (sp.team==Team::contra),
                      "warning: Specialpoints for undetermined team");
    } // if (undetermined team)
#endif
#endif


    // splitting of points in Announcements and trick specialpoints 
    // for doubling with re
    switch (sp.type) {
    case Specialpoint::Type::nospecialpoint:
      break;

    case Specialpoint::Type::caught_fox:
    case Specialpoint::Type::caught_fox_last_trick:
    case Specialpoint::Type::fox_last_trick:
    case Specialpoint::Type::charlie:
    case Specialpoint::Type::caught_charlie:
    case Specialpoint::Type::dolle_caught_dolle:
    case Specialpoint::Type::heart_trick:
    case Specialpoint::Type::doppelkopf:
      if (sp.counting_team == Team::re)
        re_trick += sp.value;
      else
        contra_trick += sp.value;
      break;

    case Specialpoint::Type::won:
      if (sp.counting_team == Team::re)
        re_played += sp.value;
      else
        contra_played += sp.value;
      break;
    case Specialpoint::Type::no90:
    case Specialpoint::Type::no60:
    case Specialpoint::Type::no30:
    case Specialpoint::Type::no0:
      if (sp.counting_team == Team::re)
        re_played += sp.value;
      else
        contra_played += sp.value;
      break;
    case Specialpoint::Type::no120_said:
      if (rule(Rule::Type::announcement_re_doubles)) {
        no120s *= sp.value;
      } else {
        if (sp.counting_team == Team::re)
          re_ann += sp.value;
        else
          contra_ann+= sp.value;
      }
      break;  
    case Specialpoint::Type::no90_said:
    case Specialpoint::Type::no60_said:
    case Specialpoint::Type::no30_said:
    case Specialpoint::Type::no0_said:
      if (sp.counting_team == Team::re)
        re_ann += sp.value;
      else
        contra_ann += sp.value;
      break;
    case Specialpoint::Type::no90_said_120_got:
    case Specialpoint::Type::no60_said_90_got:
    case Specialpoint::Type::no30_said_60_got:
    case Specialpoint::Type::no0_said_30_got:
      if (sp.counting_team == Team::re)
        re_played += sp.value;
      else
        contra_played += sp.value;
      break;
    case Specialpoint::Type::no120_reply:
    case Specialpoint::Type::no90_reply:
    case Specialpoint::Type::no60_reply:
    case Specialpoint::Type::no30_reply:
    case Specialpoint::Type::no0_reply:
      if (sp.counting_team == Team::re)
        re_ann += sp.value;
      else
        contra_ann += sp.value;
      break;
    case Specialpoint::Type::contra_won:
      if (rule(Rule::Type::announcement_contra_doubles_against_re))
        contra_played += sp.value;
      else
        contra_trick += sp.value;
      break;

    case Specialpoint::Type::solo:
      // multiplier
      break;
    case Specialpoint::Type::bock:
      // multiplier
      break;
    } // switch (sp.type)
  } // for (sp : spv)


  DEBUG_ASSERTION((no120s <= (rule(Rule::Type::knocking) ? 16 : 4)),
                  "Sum_of_Specialpoints():\n"
                  "  'no120s' = " << no120s << " > " << (rule(Rule::Type::knocking) ? 16 : 4) << "\n"
                  << spv);

  { // add the points
    int result = 0;
    switch (winner) {
    case Team::re:
      result += re_played + re_ann - contra_played;
      result *= no120s;
      result += re_trick - contra_trick; 
      break;

    case Team::contra:
      result += contra_played + contra_ann - re_played;
      result *= no120s;
      result += contra_trick - re_trick; 
      break;

    case Team::noteam:
      result += re_played - contra_played;
      result *= no120s;
      result += re_trick - contra_trick; 
      break;

    default:
      break;
    } // switch (winner)

    // multiply with the bock value
    for (auto const& sp : spv) {
      if (sp.type == Specialpoint::Type::bock)
        result *= sp.value;
    }

    return result;
  } // add the points
} // int sum(Specialpoints, Team winner)


/** -> result
 **
 ** @param    sp    special point to translate
 **
 ** @return   translation of the special point
 **/
string
gettext(Specialpoint::Type const sp)
{
  return gettext("Specialpoint::" + to_string(sp));
} // string gettext(Specialpoint::Type sp)

/** -> result
 **
 ** @param    sp   special point
 **
 ** @return   the translation of the specialpoint
 **/
string
gettext(Specialpoint const& sp)
{
  return gettext(sp.type);
} // string gettext(Specialpoints special_points)
