/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include "../basetypes.h"
class Game;

/**
 ** a structure for the management of specialpoints in tricks, gamepoints and
 ** gamepointstable
 **/
struct Specialpoint {
    enum class Type {
      nospecialpoint,
      caught_fox,
      caught_fox_last_trick,
      fox_last_trick,
      //double_fox_last_trick,
      charlie,
      caught_charlie,
      //double_charlie,
      //caught_double_charlie,
      dolle_caught_dolle,
      heart_trick,
      doppelkopf,
      won,
      no90,
      no60,
      no30,
      no0,
      no120_said,
      no90_said,
      no60_said,
      no30_said,
      no0_said,
      no90_said_120_got,
      no60_said_90_got,
      no30_said_60_got,
      no0_said_30_got,
      no120_reply,
      no90_reply,
      no60_reply,
      no30_reply,
      no0_reply,
      contra_won,
      solo,
      bock
    }; // enum class Type
    static constexpr Type types[] = {
      Type::nospecialpoint,
      Type::caught_fox,
      Type::caught_fox_last_trick,
      Type::fox_last_trick,
      //Type::double_fox_last_trick,
      Type::charlie,
      Type::caught_charlie,
      //Type::double_charlie,
      //Type::caught_double_charlie,
      Type::dolle_caught_dolle,
      Type::heart_trick,
      Type::doppelkopf,
      Type::won,
      Type::no90,
      Type::no60,
      Type::no30,
      Type::no0,
      Type::no120_said,
      Type::no90_said,
      Type::no60_said,
      Type::no30_said,
      Type::no0_said,
      Type::no90_said_120_got,
      Type::no60_said_90_got,
      Type::no30_said_60_got,
      Type::no0_said_30_got,
      Type::no120_reply,
      Type::no90_reply,
      Type::no60_reply,
      Type::no30_reply,
      Type::no0_reply,
      Type::contra_won,
      Type::solo,
      Type::bock
    };
    Specialpoint(Type type, Team team) noexcept;
    Specialpoint(Type type, Team team, Team counting_team) noexcept;
    Specialpoint(Type type, int value) noexcept;
    explicit Specialpoint(istream& istr);

    bool is_valid(vector<Team> const& team) const;

    Type type = Type::nospecialpoint;
    Team team = Team::noteam; // team which has made the point
    Team counting_team = Team::noteam; // team, which gets the point
    // the player who get the special point
    // (i.e. the player who has caught a fox)
    unsigned player_get_no = UINT_MAX;
    // the player who has 'lost' the special point
    // (i.e. the player whose fox has been caught)
    unsigned player_of_no = UINT_MAX;
    // the value of the specialpoint (used for bock rounds)
    int value = 0;
}; // struct Specialpoint

using Specialpoints = vector<Specialpoint>;


bool is_winning(Specialpoint::Type sp) noexcept;
bool is_announcement(Specialpoint::Type sp) noexcept;

int value(Specialpoint::Type sp) noexcept;
string to_string(Specialpoint::Type sp) noexcept;
Specialpoint::Type specialpoint_type_from_string(string const& name);
ostream& operator<<(ostream& ostr, Specialpoint::Type sp);
ostream& operator<<(ostream& ostr, Specialpoint const& sp);
ostream& operator<<(ostream& ostr, Specialpoints const& spv);

#if 0
// sets the counting team
Specialpoints& counting_team_set(Specialpoints& sp, Team const winner);
#endif

int sum(Specialpoints const& spv, Team winner, Game const& g);

string gettext(Specialpoint::Type sp);
string gettext(Specialpoint const& sp);
