/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include "../card/trick.h"
#include "tricks_iterator.h"
class Game;

/**
 ** The tricks of a game
 **/
class GameTricks {
  friend class Game;
  public:
    using iterator = GameTricksIterator;
    using const_iterator = GameTricksConstIterator;

    explicit GameTricks(Game& game) noexcept;
    GameTricks(GameTricks const& tricks, Game& game);

    GameTricks(GameTricks const&) = delete;
    GameTricks& operator=(GameTricks const&) = delete;

    void self_check() const;

    bool empty() const noexcept;
    unsigned size() const noexcept;
    unsigned max_size() const;

    iterator begin() noexcept;
    iterator end() noexcept;
    const_iterator begin() const noexcept;
    const_iterator end() const noexcept;

    Trick& trick(unsigned p);
    Trick const& trick(unsigned p) const;

    void add(Trick trick);

    Trick const& current() const;
    Trick& current();
    unsigned tricks_in_trickpiles() const;
    unsigned current_no() const;
    unsigned real_current_no() const noexcept;
    unsigned remaining_no() const;
    unsigned real_remaining_no() const;
    bool is_last_trick() const;
    unsigned trump_tricks_no() const;

    bool has_trick(Team team) const;
    bool has_trick(Player const& player) const;
    unsigned number_of_tricks(Player const& player) const;
    unsigned number_of_tricks_till_trick(Player const& player,
                                         unsigned trickno) const;
    unsigned number_of_tricks(Team team) const;

    void write(ostream& ostr) const;

    Signal<void(Trick const&)>& signal_trick_open() noexcept;
    Signal<void(Trick const&)>& signal_trick_full() noexcept;
    Signal<void(Trick const&)>& signal_trick_move_in_pile() noexcept;
    Signal<void(Trick const&)>& signal_trick_close() noexcept;

  private:
    Game* const game_;
    vector<Trick> tricks_;

    Signal<void(Trick const&)> signal_trick_open_;
    Signal<void(Trick const&)> signal_trick_full_;
    Signal<void(Trick const&)> signal_trick_move_in_pile_;
    Signal<void(Trick const&)> signal_trick_close_;
}; // class GameTricks
