/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#pragma once

#include "../basetypes.h"

#include "setting.h"

/**
 ** a theme for settings
 **
 ** @author   Diether Knof
 **/
class Setting::Theme {
  friend class Setting;

  public:
  Theme() = delete;
  explicit Theme(string name);
  Theme(Theme const& theme) = default;
  Theme& operator=(Theme const& theme) = default;

  ~Theme() = default;


  string const& name() const;

  // set the bool setting
  void set(Type::Bool type, bool value);
  // set the unsigned setting
  void set(Type::Unsigned type, unsigned value);
  // set the string setting
  void set(Type::String type, string const& value);

  // check whether the theme can be applied
  bool can_be_applied() const;

  private:
  // the name of the theme
  string name_;
  // boolean settings
  map<Type::Bool, bool> bool_;
  // unsigned settings
  map<Type::Unsigned, unsigned> unsigned_;
  // string settings
  map<Type::String, string> string_;
}; // class Setting::Theme
