/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 * 
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "setting_type.h"

constexpr SettingType::Bool SettingType::bool_list[];
constexpr SettingType::Unsigned SettingType::unsigned_list[];
constexpr SettingType::String SettingType::string_list[];
constexpr SettingType::StringConst SettingType::string_const_list[];
constexpr SettingType::CardsOrder SettingType::cards_order_list[];

/** -> result
 **
 ** @param    type   the type (bool)
 **
 ** @return   name of the type
 **/
string
to_string(SettingType::Bool const type)
{
  switch(type) {
  case SettingType::automatic_savings:
    return "automatic savings";
    (void)_("Setting::automatic savings");
  case SettingType::save_party_changes:
    return "save party changes";
    (void)_("Setting::save party changes");
  case SettingType::additional_party_settings:
    return "additional party settings";
    (void)_("Setting::additional party settings");
  case SettingType::show_splash_screen:
    return "show splash screen";
    (void)_("Setting::show splash screen");
  case SettingType::splash_screen_transparent:
    return "splash screen transparent";
    (void)_("Setting::splash screen transparent");
  case SettingType::show_bug_report_button_in_game_finished_window:
    return "show bug report button in game finished window";
    (void)_("Setting::show bug report button in game finished window");
  case SettingType::save_bug_reports_on_desktop:
    return "save bug reports on desktop";
    (void)_("Setting::save bug reports on desktop");
  case SettingType::sound:
    return "sound";
    (void)_("Setting::sound");
  case SettingType::automatic_card_suggestion:
    return "automatic card suggestion";
    (void)_("Setting::automatic card suggestion");
  case SettingType::announce_swines_automatically:
    return "announce swines automatically";
    (void)_("Setting::announce swines automatically");
  case SettingType::show_if_valid:
    return "show if valid";
    (void)_("Setting::show if valid");
  case SettingType::emphasize_valid_cards:
    return "emphasize valid cards";
    (void)_("Setting::emphasize valid cards");
  case SettingType::announce_in_table:
    return "announce in table";
    (void)_("Setting::announce in table");
  case SettingType::show_all_hands:
    return "show all hands";
    (void)_("Setting::show all hands");
  case SettingType::show_ai_information_hands:
    return "show ai information hands";
    (void)_("Setting::show ai information hands");
  case SettingType::show_ai_information_teams:
    return "show ai information teams";
    (void)_("Setting::show ai information teams");
  case SettingType::show_trickpiles_points:
    return "show trickpiles points";
    (void)_("Setting::show trickpiles points");
  case SettingType::show_known_teams_in_game:
    return "show known teams in game";
    (void)_("Setting::show known teams in game");
  case SettingType::show_soloplayer_in_game:
    return "show soloplayer in game";
    (void)_("Setting::show soloplayer in game");
  case SettingType::show_full_trick_window:
    return "show full trick window";
    (void)_("Setting::show full trick window");
  case SettingType::show_full_trick_window_if_special_points:
    return "show full trick window if special points";
    (void)_("Setting::show full trick window if special points");
  case SettingType::close_full_trick_automatically:
    return "close full trick automatically";
    (void)_("Setting::close full trick automatically");
  case SettingType::show_gametype_window:
    return "show gametype window";
    (void)_("Setting::show gametype window");
  case SettingType::close_gametype_window_automatically:
    return "close gametype window automatically";
    (void)_("Setting::close gametype window automatically");
  case SettingType::show_marriage_window:
    return "show marriage window";
    (void)_("Setting::show marriage window");
  case SettingType::close_marriage_window_automatically:
    return "close marriage window automatically";
    (void)_("Setting::close marriage window automatically");
  case SettingType::show_genscher_window:
    return "show genscher window";
    (void)_("Setting::show genscher window");
  case SettingType::close_genscher_window_automatically:
    return "close genscher window automatically";
    (void)_("Setting::close genscher window automatically");
  case SettingType::show_announcement_window:
    return "show announcement window";
    (void)_("Setting::show announcement window");
  case SettingType::close_announcement_window_automatically:
    return "close announcement window automatically";
    (void)_("Setting::close announcement window automatically");
  case SettingType::show_swines_window:
    return "show swines window";
    (void)_("Setting::show swines window");
  case SettingType::close_swines_window_automatically:
    return "close swines window automatically";
    (void)_("Setting::close swines window automatically");
  case SettingType::rotate_trick_cards:
    return "rotate trick cards";
    (void)_("Setting::rotate trick cards");
  case SettingType::own_hand_on_table_bottom:
    return "own hand on table bottom";
    (void)_("Setting::own hand on table bottom");
  case SettingType::original_cards_size:
    return "original cards size";
    (void)_("Setting::original cards size");
  case SettingType::ui_main_window_fullscreen:
    return "ui main window fullscreen";
    (void)_("Setting::ui main window fullscreen");
  } // switch(type)

  return "";
} // string to_string(SettingType::Bool type)

/** @param    type   type to translate
 **
 ** @return   translation of the type
 **/
string
gettext(SettingType::Bool const type)
{
  return gettext("Setting::" + to_string(type));
} // string gettext(SettingType::Bool type)

/** write the setting type in the output stream
 **
 ** @param    ostr   output stream
 ** @param    type    type to write
 **
 ** @return   output stream
 **/
ostream&
operator<<(ostream& ostr, SettingType::Bool type)
{ return (ostr << to_string(type)); }

/** -> result
 **
 ** @param    type   the type (unsigned)
 **
 ** @return   name of the type
 **/
string
to_string(SettingType::Unsigned const type)
{
  switch(type) {
#ifdef USE_THREADS
  case Setting::threads_max:
    return "max threads";
    (void)_("Setting::max threads");
#endif
  case SettingType::card_play_delay:
    return "card play delay";
    (void)_("Setting::card play delay");
  case SettingType::full_trick_close_delay:
    return "full trick close delay";
    (void)_("Setting::full trick close delay");
  case SettingType::gametype_window_close_delay:
    return "gametype window close delay";
    (void)_("Setting::gametype window close delay");
  case SettingType::marriage_window_close_delay:
    return "marriage window close delay";
    (void)_("Setting::marriage window close delay");
  case SettingType::genscher_window_close_delay:
    return "genscher window close delay";
    (void)_("Setting::genscher window close delay");
  case SettingType::announcement_window_close_delay:
    return "announcement window close delay";
    (void)_("Setting::announcement window close delay");
  case SettingType::swines_window_close_delay:
    return "swines window close delay";
    (void)_("Setting::swines window close delay");
  case SettingType::table_rotation:
    return "table rotation";
    (void)_("Setting::table rotation");
  case SettingType::cards_height:
    return "cards height";
    (void)_("Setting::cards height");
  case SettingType::ui_main_window_pos_x:
    return "ui main window pos x";
    (void)_("Setting::ui main window pos x");
  case SettingType::ui_main_window_pos_y:
    return "ui main window pos y";
    (void)_("Setting::ui main window pos y");
  case SettingType::ui_main_window_width:
    return "ui main window width";
    (void)_("Setting::ui main window width");
  case SettingType::ui_main_window_height:
    return "ui main window height";
    (void)_("Setting::ui main window height");
  } // switch(type)

  return "";
} // string to_string(SettingType::Unsigned type)

/** @param    type   type to translate
 **
 ** @return   translation of the type
 **/
string
gettext(SettingType::Unsigned const type)
{
  return gettext("Setting::" + to_string(type));
} // string gettext(SettingType::Unsigned type)

/** write the setting type in the output stream
 **
 ** @param    ostr   output stream
 ** @param    type    type to write
 **
 ** @return   output stream
 **/
ostream&
operator<<(ostream& ostr, SettingType::Unsigned type)
{ return (ostr << to_string(type)); }

/** -> result
 **
 ** @param    type   the type (string)
 **
 ** @return   name of the type
 **/
string
to_string(SettingType::String const type)
{
  switch(type) {
  case SettingType::name:
    return "name";
    (void)_("Setting::name");
  case SettingType::language:
    return "language";
    (void)_("Setting::language");
  case SettingType::cardset:
    return "cardset";
    (void)_("Setting::cardset");
  case SettingType::cards_back:
    return "cards back";
    (void)_("Setting::cards back");
  case SettingType::iconset:
    return "iconset";
    (void)_("Setting::iconset");
  case SettingType::background:
    return "background";
    (void)_("Setting::background");
  case SettingType::name_font:
    return "name font";
    (void)_("Setting::name font");
  case SettingType::name_font_color:
    return "name font color";
    (void)_("Setting::name font color");
  case SettingType::name_active_font_color:
    return "name active font color";
    (void)_("Setting::name active font color");
  case SettingType::name_reservation_font_color:
    return "name reservation font color";
    (void)_("Setting::name reservation font color");
  case SettingType::trickpile_points_font:
    return "trickpile points font";
    (void)_("Setting::trickpile points font");
  case SettingType::trickpile_points_font_color:
    return "trickpile points font color";
    (void)_("Setting::trickpile points font color");
  case SettingType::poverty_shift_arrow_color:
    return "poverty shift arrow color";
    (void)_("Setting::poverty shift arrow color");
#ifdef USE_SOUND_COMMAND
  case SettingType::play_sound_command:
    return "play sound command";
    (void)_("Setting::play sound command");
#endif
  case SettingType::browser_command:
    return "browser command";
    (void)_("Setting::browser command");
  } // switch(type)

  return "";
} // string to_string(SettingType::String type)

/** @param    type   type to translate
 **
 ** @return   translation of the type
 **/
string
gettext(SettingType::String const type)
{
  return gettext("Setting::" + to_string(type));
} // string gettext(SettingType::String type)

/** write the setting type in the output stream
 **
 ** @param    ostr   output stream
 ** @param    type    type to write
 **
 ** @return   output stream
 **/
ostream&
operator<<(ostream& ostr, SettingType::String type)
{ return (ostr << to_string(type)); }

/** -> result
 **
 ** @param    type   the type (string const)
 **
 ** @return   name of the type
 **/
string
to_string(SettingType::StringConst const type)
{
  switch(type) {
  case SettingType::cardset_license:
    return "cardset license";
    (void)_("Setting::cardset license");
  case SettingType::iconset_license:
    return "iconset license";
    (void)_("Setting::iconset license");
  case SettingType::public_data_directory:
    return "public data directory";
    (void)_("Setting::public data directory");
  case SettingType::private_data_directory:
    return "private data directory";
    (void)_("Setting::private data directory");
  case SettingType::settings_file:
    return "settings file";
    (void)_("Setting::settings file");
  case SettingType::logo_file:
    return "logo file";
    (void)_("Setting::logo file");
  case SettingType::icon_file:
    return "icon file";
    (void)_("Setting::icon file");
  case SettingType::graphic_extension:
    return "graphic extension";
    (void)_("Setting::graphic extension");
  case SettingType::sound_extension:
    return "sound extension";
    (void)_("Setting::sound extension");
  case SettingType::cardsets_directory:
    return "cardsets directory";
    (void)_("Setting::cardsets directory");
  case SettingType::cards_directory:
    return "cards directory";
    (void)_("Setting::cards directory");
  case SettingType::cards_back_directory:
    return "cards back directory";
    (void)_("Setting::cards back directory");
  case SettingType::icons_from_cardset_directory:
    return "icons from cardsets directory";
    (void)_("Setting::icons from cardsets directory");
  case SettingType::iconsets_directory:
    return "iconsets directory";
    (void)_("Setting::iconsets directory");
  case SettingType::icons_directory:
    return "icons directory";
    (void)_("Setting::icons directory");
  case SettingType::backgrounds_directory:
    return "backgrounds directory";
    (void)_("Setting::backgrounds directory");
  case SettingType::sounds_directory:
    return "sounds directory";
    (void)_("Setting::sounds directory");
  case SettingType::ai_directory:
    return "ai directory";
    (void)_("Setting::ai directory");
  case SettingType::parties_directory:
    return "parties directory";
    (void)_("Setting::parties directory");
  case SettingType::rules_directory:
    return "rules directory";
    (void)_("Setting::rules directory");
  case SettingType::settings_directory:
    return "settings directory";
    (void)_("Setting::settings directory");
  case SettingType::manual_directory:
    return "manual directory";
    (void)_("Setting::manual directory");
  } // switch(type)

  return "";
} // string to_string(SettingType::StringConst type)

/** @param    type   type to translate
 **
 ** @return   translation of the type
 **/
string
gettext(SettingType::StringConst const type)
{
  return gettext("Setting::" + to_string(type));
} // string gettext(SettingType::StringConst type)

/** write the setting type in the output stream
 **
 ** @param    ostr   output stream
 ** @param    type    type to write
 **
 ** @return   output stream
 **/
ostream&
operator<<(ostream& ostr, SettingType::StringConst type)
{ return (ostr << to_string(type)); }

/** -> result
 **
 ** @param    type   the type (cards order)
 **
 ** @return   name of the type
 **/
string
to_string(SettingType::CardsOrder const type)
{
  switch(type) {
  case SettingType::cards_order:
    return "cards order";
    (void)_("Setting::cards order");
  } // switch(type)

  return "";
} // string to_string(SettingType::CardsOrder type)

/** @param    type   type to translate
 **
 ** @return   translation of the type
 **/
string
gettext(SettingType::CardsOrder const type)
{
  return gettext("Setting::" + to_string(type));
} // string gettext(SettingType::CardsOrder type)

/** write the setting type in the output stream
 **
 ** @param    ostr   output stream
 ** @param    type    type to write
 **
 ** @return   output stream
 **/
ostream&
operator<<(ostream& ostr, SettingType::CardsOrder type)
{ return (ostr << to_string(type)); }
