/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#pragma once

struct SettingType {
  ENUM_WITH_STATIC_LIST(Bool, bool_list, 0,
                        // save nothing (automatically)
                        automatic_savings,
                        save_party_changes,
                        additional_party_settings,
                        show_splash_screen,
                        splash_screen_transparent,
                        show_bug_report_button_in_game_finished_window,
                        save_bug_reports_on_desktop,
                        sound,
                        automatic_card_suggestion,
                        announce_swines_automatically,

                        show_if_valid,
                        emphasize_valid_cards,
                        announce_in_table,
                        show_all_hands,
                        show_ai_information_hands,
                        show_ai_information_teams,
                        show_trickpiles_points,
                        show_known_teams_in_game,
                        show_soloplayer_in_game,

                        show_full_trick_window,
                        show_full_trick_window_if_special_points,
                        close_full_trick_automatically,

                        show_gametype_window,
                        close_gametype_window_automatically,
                        show_marriage_window,
                        close_marriage_window_automatically,
                        show_genscher_window,
                        close_genscher_window_automatically,
                        show_announcement_window,
                        close_announcement_window_automatically,
                        show_swines_window,
                        close_swines_window_automatically,

                        rotate_trick_cards,

                        own_hand_on_table_bottom,

                        original_cards_size,

                        ui_main_window_fullscreen,
                        );

  ENUM_WITH_STATIC_LIST(Unsigned, unsigned_list, ui_main_window_fullscreen + 1,
#ifdef USE_THREADS
                        threads_max,
#endif
                        card_play_delay,

                        full_trick_close_delay,

                        gametype_window_close_delay,
                        marriage_window_close_delay,
                        genscher_window_close_delay,
                        announcement_window_close_delay,
                        swines_window_close_delay,

                        table_rotation,

                        cards_height,

                        ui_main_window_pos_x,
                        ui_main_window_pos_y,
                        ui_main_window_width,
                        ui_main_window_height,
                        );

  ENUM_WITH_STATIC_LIST(String, string_list, ui_main_window_height + 1,
                        name,
                        language,
                        cardset,
                        cards_back,
                        iconset,
                        background,
                        name_font,
                        name_font_color,
                        name_active_font_color,
                        name_reservation_font_color,
                        trickpile_points_font,
                        trickpile_points_font_color,
                        poverty_shift_arrow_color,
#ifdef USE_SOUND_COMMAND
                        play_sound_command,
#endif
                        browser_command,
                       );

  ENUM_WITH_STATIC_LIST(StringConst, string_const_list, browser_command + 1,
                        cardset_license,
                        iconset_license,
                        public_data_directory,
                        private_data_directory,
                        settings_file,
                        logo_file,
                        icon_file,
                        graphic_extension,
                        sound_extension,
                        cardsets_directory,
                        cards_directory,
                        cards_back_directory,
                        icons_from_cardset_directory,
                        iconsets_directory,
                        icons_directory,
                        backgrounds_directory,
                        sounds_directory,
                        ai_directory,
                        parties_directory,
                        rules_directory,
                        settings_directory,
                        manual_directory);
  ENUM_WITH_STATIC_LIST(CardsOrder, cards_order_list, manual_directory + 1,
                        cards_order);
}; // struct Type

// conversion in a string
string to_string(SettingType::Bool type);
string gettext(SettingType::Bool type);
ostream& operator<<(ostream& ostr, SettingType::Bool type);

string to_string(SettingType::Unsigned type);
string gettext(SettingType::Unsigned type);
ostream& operator<<(ostream& ostr, SettingType::Unsigned type);

string to_string(SettingType::String type);
string gettext(SettingType::String type);
ostream& operator<<(ostream& ostr, SettingType::String type);

string to_string(SettingType::StringConst type);
string gettext(SettingType::StringConst type);
ostream& operator<<(ostream& ostr, SettingType::StringConst type);

string to_string(SettingType::CardsOrder type);
string gettext(SettingType::CardsOrder type);
ostream& operator<<(ostream& ostr, SettingType::CardsOrder type);
