/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#include "party_points.h"

#include "../party/party.h"
#include "../game/game_summary.h"

namespace OS_NS {

/** constructor
 **
 ** @param    ostr   output stream
 **/
PartyPoints::PartyPoints(ostream* ostr) :
  ostr(ostr)
{
  Game::signal_open().connect(*this, &PartyPoints::game_open, this->disconnector_);
}

/** destructor
 **/
PartyPoints::~PartyPoints() = default;

/** the game is opened
 **
 ** @param    game   game that is opened
 **/
void
PartyPoints::game_open(Game const& game)
{
  ::party->game().signal_finish().connect(*this, &PartyPoints::game_finish, this->disconnector_);
} // void PartyPoints::game_open(Game game)

/** the game is finished
 ** show the points
 **
 ** @param    game   finished game
 **/
void
PartyPoints::game_finish()
{
  auto const& party = *::party;
  auto const& game_summary = party.last_game_summary();

  *this->ostr << setw(5) << party.gameno() << ": ";
  for (auto const& p : party.players())
    *this->ostr << setw(8) << game_summary.points(p);
  *this->ostr << '\n';
} // void PartyPoints::game_finish(Game game)

} // namespace OS_NS
