/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#include "seed.h"

#include "../party/party.h"
#include "../game/game.h"
#include "../player/player.h"

#include <chrono>
#include <ctime>
#include <iomanip>

namespace OS_NS {

/** constructor
 **
 ** @param    ostr   output stream
 **/
Seed::Seed(ostream* ostr) :
  ostr(ostr)
{
  Game::signal_open().connect(*this, &Seed::game_open, this->disconnector_);
}

/** destructor
 **/
Seed::~Seed()
{
  if (   this->ostr
      && (this->ostr != &cout)
      && (this->ostr != &cerr)
      && (this->ostr != &clog))
    delete this->ostr;
} // Seed::~Seed()

/** the game is opened
 ** show the seed and the startplayerno
 **
 ** @param    game   game that is opened
 **/
void
Seed::game_open(Game const& game)
{
  ::party->game().signal_start().connect(*this, &Seed::game_start, this->disconnector_);

  auto now = std::chrono::system_clock::to_time_t(std::chrono::system_clock::now());
  *(this->ostr) << "seed: " << setw(8) << game.seed()
    << " (" << game.startplayer().no() << ")"
    << "\t"
    << ctime(&now)
    << '\n';
} // void Seed::game_open(Game game)

/** the game is opened
 ** show the gametype (and the soloplayernumber)
 **/
void
Seed::game_start()
{
  auto const& game = ::party->game();
  *this->ostr << "  " << game.type();
  if (is_solo(game.type()))
    *this->ostr << "\t"
      << "(soloplayer: "
      << game.players().soloplayer().no()
      << ")";
  *this->ostr << '\n';
} // void Seed::game_start(Game game)

} // namespace OS_NS
