/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include "../player/player.h"
class Party;
using PartyPlayer = Player;
class Human;
#include "players_iterator.h"

/**
 ** The players of a party
 **/
class PartyPlayers {
  public:
    using Player = PartyPlayer;
    using iterator = PartyPlayersIterator;
    using const_iterator = PartyPlayersConstIterator;

    explicit PartyPlayers(Party& party);
    PartyPlayers(Party& party, istream& istr);

    PartyPlayers() = delete;
    PartyPlayers(PartyPlayers const&) = delete;
    PartyPlayers& operator=(PartyPlayers const&) = delete;

    // signals
    Signal<void(Player const&, Player const&)>& signal_players_swapped();
    Signal<void(Player const&)>& signal_player_changed();

    bool empty() const;
    unsigned size() const;

    iterator begin();
    iterator end();
    const_iterator begin() const;
    const_iterator end() const;

    // a player
    Player& player(unsigned p);
    Player const& player(unsigned p) const;
    Player& operator[](unsigned p);
    Player const& operator[](unsigned p) const;
    // the position of a player
    unsigned no(Player const& player) const;

    // number of human players
    unsigned count_humans() const;
    // the human player (if there is exactly one)
    Human* human_player();
    Human const* human_player() const;

    void set(unsigned n, unique_ptr<Player> player);
    // swap two players
    void swap(unsigned n0, unsigned n1);

    void set_type(Player& player, Player::Type new_type);
    void set_type(unsigned n, Player::Type new_type);

    void write(ostream& ostr) const;
    void write_short(ostream& ostr) const;

    bool load();
    void save() const;

  private:
    Party* const party_;
    // players
    vector<unique_ptr<Player>> players_;

    // signals
    Signal<void(Player const&, Player const&)> signal_players_swapped_;
    Signal<void(Player const&)> signal_player_changed_;
}; // class PartyPlayers
