/**********************************************************************
 *
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "best_winning_card.h"

#include "../team_information.h"
#include "../../../card/trick.h"

#include "../ai.h"
#include "../heuristics.h"

/** constructor
 **/
Heuristic::BestWinningCard::BestWinningCard(Ai const& ai) :
  Heuristic(ai, Aiconfig::Heuristic::best_winning)
{ }

/** destructor
 **/
Heuristic::BestWinningCard::~BestWinningCard() = default;

/** @return  card to play or empty card
 **/
Card
Heuristic::BestWinningCard::get_card(Trick const& trick)
{
  this->rationale_.clear();
  if (!(   !this->condition_startcard(trick)
        && !this->condition_high_winner_card_of_own_team(trick)
       ))
    return {};

  return Heuristics::best_winning_card(trick, *this->ai_);
}

/** @return   the own team is the winner and has jabbed a high card
 **/
bool
Heuristic::BestWinningCard::condition_high_winner_card_of_own_team(Trick const& trick)
{
  auto const& winnerplayer = trick.winnerplayer();
  if (!this->guessed_same_team(winnerplayer)) {
    this->rationale_.add(_("Heuristic::condition::winnerplayer is of opposite team"));
    return false;
  }
  this->rationale_.add(_("Heuristic::condition::winnerplayer is of the same team"));
  auto const& winnercard = trick.winnercard();
  if (winnercard.istrump()) {
    auto const trumplimit_normal = this->value(Aiconfig::Type::trumplimit_normal);
    if (trumplimit_normal.less(winnercard)) {
      this->rationale_.add(_("Heuristic::condition::trumplimit low (%s) is less then the winnercard", _(trumplimit_normal)));
      return true;
    } else {
      this->rationale_.add(_("Heuristic::condition::trumplimit low (%s) is not less then the winnercard", _(trumplimit_normal)));
      return false;
    }
  } else { // color trick
    if (winnercard.value() != Card::ace) {
      this->rationale_.add(_("Heuristic::condition::winnercard not ace"));
      return false;
    } else if (this->cards_information().played(winnercard.tcolor()) <= trick.actcardno()) {
      this->rationale_.add(_("Heuristic::condition::winnercard is ace and the color has not been played before"));
      return true;
    } else {
      this->rationale_.add(_("Heuristic::condition::winnercard is ace and the color has already been played before"));
      return false;
    }
  }
}
