/**********************************************************************
 *
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "jab_color_trick_with_dolle.h"

#include "../ai.h"
#include "../team_information.h"
#include "../heuristics.h"
#include "../../../game/game.h"
#include "../../../party/rule.h"


/** constructor
 **/
Heuristic::JabColorTrickWithDolle::JabColorTrickWithDolle(Ai const& ai) :
  Heuristic(ai, Aiconfig::Heuristic::jab_color_trick_with_dolle)
{ }

/** destructor
 **/
Heuristic::JabColorTrickWithDolle::~JabColorTrickWithDolle() = default;

/** @return  card to play or empty card
 **/
Card
Heuristic::JabColorTrickWithDolle::get_card(Trick const& trick)
{
  this->rationale_.clear();
  if (!(   !this->condition_solo_meatless(trick)
        && !this->condition_solo_picture(trick)
        && !this->condition_startcard(trick)
        && !this->condition_last_card(trick)
        && this->condition_color_trick(trick)
        && !this->condition_first_color_run(trick)
        && this->condition_has_dolle()
        && this->condition_can_jab_with(trick, Card::dolle)
        && this->condition_opposite_team_behind(trick)
        && !this->condition_own_team_jabbed_with_trump_card_limit(trick)
        && !this->condition_can_be_jabbed_by_opposite_team(trick, Card::dolle)
       ))
    return {};

  if (trick.points() < 10) {
    this->rationale_.add(_("Heuristic::reject::not enough points in the trick"));
    return {};
  }

  this->rationale_.add(_("Heuristic::return::jab the trick with a dolle"));
  return Card::dolle;
}
