/**********************************************************************
 *
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "picture_second_best_trump.h"

#include "../../team_information.h"
#include "../../../../card/trick.h"
#include "../../../../game/game.h"
#include "../../../../party/rule.h"

#include "../../ai.h"
#include "../../heuristics.h"

/** constructor
 **/
Heuristic::PictureSecondBestTrump::PictureSecondBestTrump(Ai const& ai) :
  Heuristic(ai, Aiconfig::Heuristic::picture_secondbesttrump)
{ }

/** destructor
 **/
Heuristic::PictureSecondBestTrump::~PictureSecondBestTrump() = default;

/** @return  card to play or empty card
 **/
Card
Heuristic::PictureSecondBestTrump::get_card(Trick const& trick)
{
  this->rationale_.clear();
  if (!(   this->condition_solo_picture(trick)
        && this->condition_startcard(trick)
        && this->condition_trump_on_hand()
        && this->condition_others_trump_on_hand()
       ))
    return {};

  Ai& ai = const_cast<Ai&>(*this->ai_);
  auto const& hand = this->ai_->hand();
  if (   (ai.meatlesscolor_ == Card::nocardcolor)
      || (   !hand.contains(Card(ai.meatlesscolor_, Card::ace))
          && ai.cards_information().remaining(Card(ai.meatlesscolor_, Card::ace))))
  {
    auto const& game = trick.game();
    int maxcards = -1;
    for (auto color : game.rule().card_colors()) {
      auto const n = ai.calculate_meatless_color(color) ;
      if (n > maxcards) {
        maxcards = n;
        ai.meatlesscolor_ = color;
      }
    }
  }

  auto const card = Heuristics::play_highest_color(trick, ai,
                                                   ai.meatlesscolor_);
  if (!card)
    ai.meatlesscolor_ = Card::nocardcolor;
  if (hand.count(card.color()) == 1) //last card of this color
    ai.meatlesscolor_ = Card::nocardcolor; // trigger new calculation

  return card;
}

/** @return   whether the player has trump on his hand
 **/
bool
Heuristic::PictureSecondBestTrump::condition_trump_on_hand()
{
  return this->hand().contains(Card::Color::trump);
}

/** @return   whether the other players have trump on his hand
 **/
bool
Heuristic::PictureSecondBestTrump::condition_others_trump_on_hand()
{
  return this->ai_->cards_information().remaining_trumps_others();
}
