/**********************************************************************
 *
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "start_with_low_trump.h"

#include "../team_information.h"
#include "../../../card/trick.h"
#include "../../../game/game.h"
#include "../../../party/rule.h"

#include "../ai.h"
#include "../heuristics.h"

/** constructor
 **/
Heuristic::StartWithLowTrump::StartWithLowTrump(Ai const& ai) :
  Heuristic(ai, Aiconfig::Heuristic::start_with_low_trump)
{ }

/** destructor
 **/
Heuristic::StartWithLowTrump::~StartWithLowTrump() = default;

/** @return  card to play or empty card
 **/
Card
Heuristic::StartWithLowTrump::get_card(Trick const& trick)
{
  this->rationale_.clear();
  if (!(   this->condition_startcard(trick)
        && !this->condition_solo_game(trick)
        && !this->condition_partner_guessed(trick)))
    return {};

  auto const& hand = this->player().hand();
  auto const& ai = *this->ai_;

  auto const card
    = HandCard(hand,
               Heuristics::lowest_best_trump_card(trick, hand, ai));

  if (!card.less(ai.value(Aiconfig::Type::trumplimit_normal))) {
    this->rationale_.add(_("Heuristic::reject::trump too high"));
    return {};
  }
  if (card.value() >= 10) {
    this->rationale_.add(_("Heuristic::reject::selected trump has %i points", card.value()));
    return {};
  }

  this->rationale_.add(_("Heuristic::return::lowest best trump: %s", _(card)));
  return card;
}
