/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#pragma once

#include "ai.h"
#include "virtual_games_interface.h"

class VirtualGamesAi : public VirtualGamesInterface {
  public:	
    VirtualGamesAi() = delete;
    VirtualGamesAi( Ai const& a ): VirtualGamesInterface(), ai_( &a ){}
    VirtualGamesAi(VirtualGamesAi const& ai) = delete;
    VirtualGamesAi& operator=(VirtualGamesAi const& ai) = delete;
    ~VirtualGamesAi() override = default;

    CardsInformation const& cards_information() const override
    { return this->ai_->cards_information(); }
    TeamInformation const& team_information() const override
    { return this->ai_->team_information(); }
    
    Player const& player() const override
    { return *this->ai_; }
    Hand const& handofplayer( Player const& player ) const override
    { return this->ai_->handofplayer( player ); }
    Team teamofplayer( Player const& player ) const override
    { return this->ai_->teamofplayer( player ); }
 
    // create a virtual game with virtual players (uses cards information and team information)
    unique_ptr<Game> create_virtual_game(vector<unique_ptr<Player>>& players) const override;
    vector<unique_ptr<Player>> create_virtual_ais() const override;

    Hand const& hand() const override
    { return this->ai_->hand(); }
    Team team() const override
    { return this->ai_->team(); }
    Game const& game() const override
    { return this->ai_->game(); }
    unsigned last_trick_to_calculate() const override
    { return this->ai_->last_trick_to_calculate(); }

    bool value(Aiconfig::Type::Bool type) const override
    { return this->ai_->value( type ); }
    int value(Aiconfig::Type::Int type) const override
    { return this->ai_->value( type ); }
    Card value(Aiconfig::Type::Card type) const override
    { return this->ai_->value( type ); }

    unsigned color_runs(Card::TColor c) const override
    { return this->ai_->color_runs( c ); }
    bool colorjabbed(Card::TColor c) const override
    { return this->ai_->colorjabbed( c ); }

    bool jabbedbyownteam(Card::TColor c) const override
    { return this->ai_->jabbedbyownteam( c ); }
    bool jabbedbyotherteam(Card::TColor c) const override
    { return this->ai_->jabbedbyotherteam( c ); }
    
    Card trump_card_limit() const override
    { return this->ai_->trump_card_limit(); }
  private:
    Ai const* ai_;	
}; // class VirtualGamesAi : public VirtualGameInterface
