/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

enum class AiconfigHeuristic {
  error = -1,
  no_heuristic = 0,
  manual,
  bug_report,
  only_valid_card,
  play_to_marry,
  play_to_get_married,
  choose_ace,
  choose_ten,
  choose_single_ace,
  choose_for_color_trick,
  jab_color_over_fox,

  start_with_color,
  start_with_low_color,
  start_with_low_trump,
  retry_color,
  play_color_for_partner,
  try_color_for_partner,
  play_color_for_partner_ace,
  play_bad_color,
  jab_color_trick_with_dolle,
  save_dolle,
  pfund_in_first_color_run,
  serve_color_trick,
  serve_trump_trick,
  choose_pfund,
  choose_pfund_poverty,
  choose_pfund_before_partner,
  jab_for_ace,
  create_fehl,
  best_winning,
  low_high,
  play_trump,
  play_for_team,
  jab_fox,
  jab_for_doppelkopf,
  try_for_doppelkopf,
  play_for_partner_worries,
  partner_backhand_draw_trump,
  draw_trump,
  play_to_jab_later,
  play_highest_color_card_in_game,
  get_trick_for_announcement,
  grab_trick,
  let_trick_pass,

  poverty_special_play_pfund,
  poverty_special_give_no_points,
  poverty_special_offer_pfund,
  poverty_re_trump_color_trick_high,
  poverty_re_play_trump,
  poverty_contra_play_color,
  poverty_contra_trump_color_trick_high,
  poverty_leave_to_partner,
  poverty_overjab_re,
  poverty_best_winning_card,

  play_color_in_solo,

  meatless_playhighestcolor,
  picture_secondbesttrump,

  color_jab_for_ace,
  color_low_high,

  choose_best_card, // for the database
};
constexpr AiconfigHeuristic aiconfig_heuristic_list[] = {
  AiconfigHeuristic::no_heuristic,
  AiconfigHeuristic::manual,
  AiconfigHeuristic::bug_report,
  AiconfigHeuristic::only_valid_card,
  AiconfigHeuristic::play_to_marry,
  AiconfigHeuristic::play_to_get_married,
  AiconfigHeuristic::choose_ace,
  AiconfigHeuristic::choose_ten,
  AiconfigHeuristic::choose_single_ace,
  AiconfigHeuristic::pfund_in_first_color_run,
  AiconfigHeuristic::choose_for_color_trick,
  AiconfigHeuristic::jab_color_over_fox,

  AiconfigHeuristic::start_with_color,
  AiconfigHeuristic::start_with_low_color,
  AiconfigHeuristic::start_with_low_trump,
  AiconfigHeuristic::retry_color,
  AiconfigHeuristic::play_color_for_partner,
  AiconfigHeuristic::try_color_for_partner,
  AiconfigHeuristic::play_color_for_partner_ace,
  AiconfigHeuristic::play_bad_color,
  AiconfigHeuristic::jab_color_trick_with_dolle,
  AiconfigHeuristic::save_dolle,
  AiconfigHeuristic::serve_color_trick,
  AiconfigHeuristic::serve_trump_trick,
  AiconfigHeuristic::choose_pfund,
  AiconfigHeuristic::choose_pfund_poverty,
  AiconfigHeuristic::choose_pfund_before_partner,
  AiconfigHeuristic::jab_for_ace,
  AiconfigHeuristic::create_fehl,
  AiconfigHeuristic::best_winning,
  AiconfigHeuristic::low_high,
  AiconfigHeuristic::play_trump,
  AiconfigHeuristic::play_for_team,
  AiconfigHeuristic::jab_fox,
  AiconfigHeuristic::jab_for_doppelkopf,
  AiconfigHeuristic::try_for_doppelkopf,
  AiconfigHeuristic::play_for_partner_worries,
  AiconfigHeuristic::partner_backhand_draw_trump,
  AiconfigHeuristic::draw_trump,
  AiconfigHeuristic::play_to_jab_later,
  AiconfigHeuristic::play_highest_color_card_in_game,
  AiconfigHeuristic::get_trick_for_announcement,
  AiconfigHeuristic::grab_trick,
  AiconfigHeuristic::let_trick_pass,

  AiconfigHeuristic::poverty_special_play_pfund,
  AiconfigHeuristic::poverty_special_give_no_points,
  AiconfigHeuristic::poverty_special_offer_pfund,
  AiconfigHeuristic::poverty_re_trump_color_trick_high,
  AiconfigHeuristic::poverty_re_play_trump,
  AiconfigHeuristic::poverty_contra_play_color,
  AiconfigHeuristic::poverty_contra_trump_color_trick_high,
  AiconfigHeuristic::poverty_leave_to_partner,
  AiconfigHeuristic::poverty_overjab_re,
  AiconfigHeuristic::poverty_best_winning_card,

  AiconfigHeuristic::play_color_in_solo,

  AiconfigHeuristic::meatless_playhighestcolor,
  AiconfigHeuristic::picture_secondbesttrump,

  AiconfigHeuristic::color_jab_for_ace,
  AiconfigHeuristic::color_low_high,

  AiconfigHeuristic::choose_best_card, // for the database
};

constexpr unsigned aiconfig_heuristic_size = static_cast<int>(AiconfigHeuristic::choose_best_card) + 1;

bool is_real(AiconfigHeuristic heuristic);
bool is_general(AiconfigHeuristic heuristic);
bool for_poverty(AiconfigHeuristic heuristic);
bool for_solo(AiconfigHeuristic heuristic);
bool for_color_solo(AiconfigHeuristic heuristic);
bool for_picture_solo(AiconfigHeuristic heuristic);
bool for_meatless_solo(AiconfigHeuristic heuristic);


string to_string(AiconfigHeuristic heuristic);
AiconfigHeuristic AiconfigHeuristic_from_name(string const& name);

ostream& operator<<(ostream& ostr, AiconfigHeuristic heuristic);
string gettext(AiconfigHeuristic heuristic);
string gettext_description(AiconfigHeuristic heuristic);
