/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include "../ai/ai.h"

class Human : public Ai {
  public:
    Human();
    explicit Human(istream& istr);
    Human(Player const& player);
    Human(Player const& player, Aiconfig const& aiconfig);
    Human(Ai const& ai);
    Human(Human const& human);
    Human& operator=(const Human& player) = delete;
    unique_ptr<Player> clone() const override;

    // desctructor
    ~Human() override = default;

    // get the reservation
    Reservation const& reservation_get(bool const is_duty = false) override;
    // the game is started
    void game_start() override;
    // make a save announcement
    Announcement announcement_request() const override;
    // get the card to play
    HandCard card_get() override;

    // update the team information
    void teaminfo_update() override;

    // 'player' shifts cards
    HandCards poverty_shift() override;
    // returns whether 'player' accepts the poverty
    bool poverty_take_accept(unsigned const cardno) override;
    // the player changes the cards of the poverty
    HandCards poverty_cards_change(HandCards const& cards) override;
    // the poverty player gets 'cards'
    void poverty_cards_get_back(HandCards const& cards) override;

    // the player has swines
    void swines_announced(Player const& player) override;

    // return a partner for a genscher
    Player const* genscher_partner() override;
}; // class Human: public Ai
