/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#include "constants.h"
#include "trickpile.h"
#include "player.h"
#include "../game/game.h"
#include "../party/rule.h"

/** constructor
 **
 ** @param    game   corresponding game
 **/
TrickPile::TrickPile(Player& player) :
  player_(&player)
{ }

/** copy constructor
 **/
TrickPile::TrickPile(TrickPile const& trickpile, Player& player) :
  player_(&player),
  tricks_(trickpile.tricks_)
{ }

/** copy operator
 **/
TrickPile&
TrickPile::operator=(TrickPile const& trickpile)
{
  this->tricks_ = trickpile.tricks_;
  return *this;
}

/** @return   iterator to first player
 **/
TrickPile::const_iterator
TrickPile::begin() const
{
  return TrickPile::const_iterator(*this, 0);
}

/** @return   iterator behind the last player
 **/
TrickPile::const_iterator
TrickPile::end() const
{
  return TrickPile::const_iterator(*this, this->size());
}

/** @return   whether there is no trick
 **/
bool
TrickPile::empty() const
{
  return this->tricks_.empty();
}

/** @return   the number of tricks
 **/
unsigned
TrickPile::size() const
{
  return this->tricks_.size();
}

/** clear all tricks
 **/
void
TrickPile::clear()
{
  this->tricks_.clear();
}

/** -> result
 **
 ** @param    t   number of the trick
 **
 ** @return   the trick with the number 't'
 **/
Trick&
TrickPile::trick(unsigned const t)
{
  DEBUG_ASSERTION(t < this->tricks_.size(),
                  "TrickPile::trick(t):\n"
                  "'t' is not valid (" << t << ")");

  return this->player_->game().tricks().trick(this->tricks_[t]);
}

/** -> result
 **
 ** @param    t   number of the trick
 **
 ** @return   the trick with the number 't'
 **/
Trick const&
TrickPile::trick(unsigned const t) const
{
  DEBUG_ASSERTION(t < this->tricks_.size(),
                  "TrickPile::trick(t):\n"
                  "'t' is not valid (" << t << ")");

  return this->player_->game().tricks().trick(this->tricks_[t]);
}

/** @return   the last trick
 **/
Trick const&
TrickPile::last_trick() const
{
  DEBUG_ASSERTION(!this->tricks_.empty(),
                  "TrickPile::last_trick():\n"
                  "The trickpile is empty");

  return this->trick(this->size() - 1);
}

/** add a trick
 **
 ** @param   trick to add
 **/
void
TrickPile::add(Trick const& trick)
{
  this->tricks_.push_back(trick.no());
}
