/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public Error as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the Error, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public Error for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public Error
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "error.h"

#include "bug_report.h"
#include "ui.h"

#include "../../misc/setting.h"

#include <gtkmm/button.h>

namespace UI_GTKMM_NS {

/** constructor
 **
 ** @param    message   message to display
 ** @param    parent    the parent object
 **/
Error::Error(string  message_, Base* const parent) :
  Base(parent),
  Gtk::MessageDialog("FreeDoko – " + _("Window::error"),
                     false,
                     Gtk::MESSAGE_ERROR, Gtk::BUTTONS_NONE,
#ifdef RELEASE
                     true
#else
                     false
#endif
                    ),
  message(std::move(message_))
  {
#ifdef GLIBMM_EXCEPTIONS_ENABLED
    try {
      this->set_icon(Gdk::Pixbuf::create_from_file(::setting.path(Setting::Type::icon_file)));
    } catch (Glib::FileError file_error) {
    } catch (Gdk::PixbufError pixbuf_error) {
    } // try
#else
    std::auto_ptr<Glib::Error> error;
    this->set_icon(Gdk::Pixbuf::create_from_file(::setting.path(Setting::Type::icon_file), error));
#endif

    // gettext: %s = contact
    this->set_message(_("BugReport::bug found, create report, %s",
                        CONTACT)
                      + "\n\n"
                      + _("BugReport::message") + ":"
                      + "\n"
                      + message);
    {
      auto bug_report_button = Gtk::manage(new Gtk::Button(_("BugReport::create")));
      bug_report_button->set_image_from_icon_name("document-new");
      bug_report_button->set_always_show_image();
      this->add_action_widget(*bug_report_button, Gtk::RESPONSE_CLOSE);
      bug_report_button->signal_clicked().connect(sigc::mem_fun(*this,
                                                                &Error::create_bug_report)
                                                 );
    }
    {
      auto quit_button = Gtk::manage(new Gtk::Button(_("quit")));
      quit_button->set_image_from_icon_name("application-exit");
      quit_button->set_always_show_image();
      this->add_action_widget(*quit_button, Gtk::RESPONSE_CLOSE);
      quit_button->signal_clicked().connect(sigc::mem_fun(*this,
                                                          &Error::quit)
                                           );
    }

    this->show_all_children();
  } // Error::Error(string message, Base* parent)

/** destructor
 **/
Error::~Error() = default;

/** show the bug report window
 **/
void
  Error::create_bug_report()
  {
    BugReport bug_report(this, this->message + "\n---\n\n");

    bug_report.run();
    this->hide();
  }

/** quit the program
 **/
void
  Error::quit()
  {
    exit(EXIT_FAILURE);
  }

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
