/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "genscher_selection.h"
#include "table.h"
#include "ui.h"
#include "main_window.h"

#include "../../game/game.h"
#include "../../player/player.h"

#include <gtkmm/box.h>
#include <gtkmm/label.h>
#include <gtkmm/grid.h>
#include <gtkmm/main.h>

namespace UI_GTKMM_NS {

/** constructor
 **
 ** @param    table      the table
 **/
GenscherSelection::GenscherSelection(Table* const table) :
  Base(table),
  Gtk::StickyDialog("FreeDoko – " + _("Window::Genscher selection"),
                    *table->ui->main_window, true),
  partner_button()
{
  this->ui->add_window(*this);

  this->signal_realize().connect(sigc::mem_fun(*this,
                                               &GenscherSelection::init));
} // GenscherSelection::GenscherSelection(Table* table)

/** destructor
 **/
GenscherSelection::~GenscherSelection() = default;

/** init the dialog
 **/
void
GenscherSelection::init()
{
  this->set_icon(this->ui->icon);
  this->set_skip_taskbar_hint();

  auto vbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 1 EX));
  vbox->set_border_width(1 EX);
  this->get_action_area()->add(*vbox);

  this->info_label = Gtk::manage(new Gtk::Label(_("Label::Genscher: Please choose your partner.")));
  vbox->add(*(this->info_label));

  auto grid = Gtk::manage(new Gtk::Grid());
  grid->set_row_homogeneous(true);
  grid->set_column_homogeneous(true);
  grid->set_row_spacing(1 EX);
  grid->set_column_spacing(1 EM);
  grid->set_border_width(1 EX);

  for (unsigned p = 0; p < this->genscher->game().playerno(); p++) {
    auto const& player = this->genscher->game().player(p);
    this->partner_button.push_back(Gtk::manage(new Gtk::Button(player.name())));
    this->partner_button.back()->signal_clicked().connect(sigc::bind<Player const* const>(sigc::mem_fun(*this, &GenscherSelection::select_partner_event), &player));
    switch(dynamic_cast<Table*>(this->parent)->position(player)) {
    case Position::north:
      grid->attach(*this->partner_button.back(),
                    1, 0, 1, 1);
      break;
    case Position::south:
      grid->attach(*this->partner_button.back(),
                    1, 2, 1, 1);
      break;
    case Position::west:
      grid->attach(*this->partner_button.back(),
                    0, 1, 1, 1);
      break;
    case Position::east:
      grid->attach(*this->partner_button.back(),
                    2, 1, 1, 1);
      break;
    case Position::center:
      DEBUG_ASSERTION(false,
                      "GenscherSelection::init()\n"
                      "  wrong position 'center' "
                      << "(" << dynamic_cast<Table*>(this->parent)->position(player) << ")");
      break;
    } // switch(this->table->position(player))
  } // for (p < this->genscher->game().playerno())
  vbox->add(*grid);

  this->no_partner_change_button = Gtk::manage(new Gtk::Button(_("Button::no partner change")));
  vbox->add(*(this->no_partner_change_button));

  this->no_partner_change_button->signal_clicked().connect(sigc::bind<Player const* const>(sigc::mem_fun(*this, &GenscherSelection::select_partner_event), nullptr));
  this->show_all_children();
} // void GenscherSelection::init()

/** show the window
 **
 ** @param    genscher   the player who has the genscher
 **/
Player const*
GenscherSelection::select_partner(Player const& genscher)
{
  this->genscher = &genscher;
  if (!this->get_realized())
    this->realize();

  // Note: nullptr is used for no selection
  this->partner = this->genscher;

  this->partner_button[genscher.no()]->hide();

  this->present();

  while (!this->ui->thrower
         && (this->partner == this->genscher)
         && this->is_visible())
    ::ui->wait();

  this->hide();

  if (this->partner == this->genscher)
    this->partner = nullptr;

  this->genscher = nullptr;

  return this->partner;
} // Player const* GenscherSelection::select_partner(Player genscher)

/** a partner has been selected
 **
 ** @param    player   the player who has been selected
 **/
void
GenscherSelection::select_partner_event(Player const* const player)
{
  this->partner = player;
} // void GenscherSelection::select_partner_event(Player const* player)

/** the name of 'player' has changed
 **
 ** @param    player   the player whose name has changed
 **/
void
GenscherSelection::name_changed(Player const& player)
{
  if (!this->get_realized())
    return ;

  dynamic_cast<Gtk::Label*>(this->partner_button[player.no()]->get_child()
                           )->set_text(player.name());
} // void GenscherSelection::name_changed(Player player)

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
