/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#ifdef USE_UI_GTKMM

#pragma once

#include "htin.h"

#include "../../card/card.h"
#include "../../card/hand.h"

class Ai;

namespace UI_GTKMM_NS {

/**
 ** the graphic representation of a sorted hand
 **
 ** @todo   use of 'Gdk::Point' and 'Gdk::Rectangle'
 **/
class Hand : public HTIN {
public:
  /**
   ** The status for checking changes (for redrawing)
   ** @todo: card suggestion
   **/
  struct Status {
    bool show_tricks = true;
    bool show_hand = false;
    bool show_ai_information_hand = false;
    Ai const* ai = nullptr;
    ::Hand hand;
  }; // struct Status

  /**
   ** The sizes for drawing a hand
   **/
  struct Geometry : public Table::Part::Geometry {
    function<int()> margin_x;
    function<int()> margin_y;
    function<int()> skip_y;
    function<double()> cards_step_x;

    Outline outline() const override;
  }; // struct Geometry : public Table::Part::Geometry

public:
  explicit Hand(Table* table, Position position);
  ~Hand() override;

  bool changed() const override;

  void draw(Cairo::RefPtr<::Cairo::Context> cr) override;
  void draw_tricks(Cairo::RefPtr<::Cairo::Context> cr);

  Geometry geometry(Position position) const;
  Geometry geometry() const;
  Outline outline() const override;

  optional<unsigned> cardno_at_position(int x, int y) const;
  Card card_at_position(int x, int y) const;

public:
  //private:
  Status current_status() const;
  Status last_status_;

private: // unused
  Hand() = delete;
  Hand(Hand const&) = delete;
  Hand& operator=(Hand const&) = delete;
}; // class Hand : public HTIN

bool operator!=(Hand::Status const& lhs, Hand::Status const& rhs);
ostream& operator<<(ostream& ostr, Hand::Status const& status);

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
