/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#ifdef USE_UI_GTKMM

#pragma once

#include "base.h"
#include "widgets/sticky_dialog.h"
#include "trick_summary.h"

class Trick;
class Player;

namespace Gtk {
class Button;
class Box;
}; // namespace Gtk

namespace UI_GTKMM_NS {
class Table;

/**
 ** the last trick window
 **/
class LastTrick : public Base, public Gtk::StickyDialog {
  public:
    explicit LastTrick(Table* table);
    ~LastTrick() override;

    // open the window and show the last trick
    void show_last_trick(::Trick const& last_trick);
    // close the window
    void close();

    // the current tricknumber
    unsigned trickno() const;
    // set the current trick number
    void set_trickno(unsigned trickno);

    // the name of a player has changed
    void name_changed(::Player const& player);

    // a key has been pressed
    bool key_press(GdkEventKey* key);

  private:
    // initialize the elements
    void init();

    // show the previous trick
    void previous_trick();
    // show the next trick
    void following_trick();

    // on hide signal
    void on_hide() override;

  private:
    // the trick shown
    unsigned last_trick_no = UINT_MAX;

    Gtk::Box* navigation_container = nullptr;
    Gtk::Button* previous_trick_button = nullptr;
    Gtk::Button* following_trick_button = nullptr;

    unique_ptr<TrickSummary> trick_summary;

  private: // unused
    LastTrick() = delete;
    LastTrick(LastTrick const&) = delete;
    LastTrick& operator=(LastTrick const&) = delete;
}; // class LastTrick : public Base, public Gtk::StickyDialog

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
