/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#ifdef USE_UI_GTKMM

#pragma once

#include "htin.h"

#include <pangomm/layout.h>

namespace UI_GTKMM_NS {

/**
 ** the graphic representation of a name
 **
 ** @todo   use of 'Gdk::Point' and 'Gdk::Rectangle'
 **/
class Name : public HTIN {
  public:
    /**
     ** The status for checking changes (for redrawing)
     **/
    struct Status {
      string name;
      bool is_active;
      string color;
      double progress;
    }; // struct Status
    /**
     ** The geometry of the name
     **/
    struct Geometry : public Table::Part::Geometry {
      Rotation rotation = Rotation::up;
      Outline outline() const override;
    }; // struct Geometry
  public:
    Name(Table* table, Position position);
    ~Name() override;

    Cairo::RefPtr<Cairo::ImageSurface> surface() override;

    bool changed() const override;

    Geometry geometry() const;
    Outline outline() const override;

    void draw(Cairo::RefPtr<::Cairo::Context> cr) override;

  private:
    int width_ = 0;
    int height_ = 0;

    Status current_status() const;
    Status last_status_;

  private: // unused
    Name() = delete;
    Name(Name const&) = delete;
    Name& operator=(Name const&) = delete;
}; // class Name : public HTIN

bool operator!=(Name::Status const& lhs, Name::Status const& rhs);

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
