/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#ifdef USE_UI_GTKMM

#pragma once

#include "base.h"
#include "widgets/sticky_dialog.h"

#include "../../game/specialpoint.h"
class Party;
class Player;
class PlayersDb;
class TLWcount;

#include <gtkmm/treemodel.h>
#include <gtkmm/treestore.h>
namespace Gtk {
class TreeView;
} // namespace Gtk
namespace UI_GTKMM_NS {
/**
 ** window with the player db
 **
 ** @todo   only update the heuristics at the end of a game,
 **         so that the human player can get no additional information
 **         (p.e. with 'only valid card')
 **/
class PlayersDB : public Base, public Gtk::StickyDialog {
  enum class Statistic {
    total,
    won,
    lost,
    percent_won
  }; // enum Statistic
  static constexpr Statistic statistic_all[] = {
    Statistic::total,
    Statistic::won, 
    Statistic::lost, 
    Statistic::percent_won, 
  };
  static string to_string(Statistic statistic);
  static string gettext(Statistic statistic);

  // model for the data
  struct PlayersDBModel : public Gtk::TreeModel::ColumnRecord {
      PlayersDBModel(unsigned playerno);

      Gtk::TreeModelColumn<Glib::ustring> type;
      vector<Gtk::TreeModelColumn<Glib::ustring> > statistic;
  }; // struct PlayersDBModel : public Gtk::TreeModel::ColumnRecord

  public:
  explicit PlayersDB(Base* parent);
  ~PlayersDB() override;

  PlayersDB() = delete;
  PlayersDB(PlayersDB const&) = delete;
  PlayersDB& operator=(PlayersDB const&) = delete;

  void party_close();
  void game_finished();

  void update_db();
  void name_changed(Player const& player);

  private:
  void init();
  void recreate_db();
  void set_statistic(Statistic statistic);

  void clear_db();

  // returns a string representing the data
  Glib::ustring statistic_data(TLWcount const& tlwcount) const;
  // returns a string representing the data
  Glib::ustring heuristic_statistic_data(unsigned total,
                                         unsigned num) const;

  // a key has been pressed
  bool on_key_press_event(GdkEventKey* key) override;

  private:
  AutoDisconnector disconnector_;

  Statistic statistic = Statistic::total;

  std::unique_ptr<PlayersDBModel> players_db_model;
  Glib::RefPtr<Gtk::TreeStore> players_db_list;
  Gtk::TreeView* players_db_treeview = nullptr;

  Gtk::TreeModel::Row ranking;
  Gtk::TreeModel::Row average_game_points;
  Gtk::TreeModel::Row average_game_trick_points;
  Gtk::TreeModel::Row games;
  Gtk::TreeModel::Row games_marriage;
  Gtk::TreeModel::Row games_poverty;
  Gtk::TreeModel::Row games_solo;
  Gtk::TreeModel::Row games_solo_color;
  Gtk::TreeModel::Row games_solo_picture;
  Gtk::TreeModel::Row games_solo_picture_single;
  Gtk::TreeModel::Row games_solo_picture_double;
  vector<Gtk::TreeModel::Row> game;
  Gtk::TreeModel::Row specialpoints;
  Gtk::TreeModel::Row specialpoints_winning;
  Gtk::TreeModel::Row specialpoints_announcement;
  std::map<Specialpoint::Type, Gtk::TreeModel::Row> specialpoint;
  Gtk::TreeModel::Row heuristics;
  Gtk::TreeModel::Row heuristics_general;
  Gtk::TreeModel::Row heuristics_poverty;
  Gtk::TreeModel::Row heuristics_solo;
  Gtk::TreeModel::Row heuristics_solo_color;
  Gtk::TreeModel::Row heuristics_solo_picture;
  Gtk::TreeModel::Row heuristics_solo_meatless;
  vector<Gtk::TreeModel::Row> heuristic;
}; // class PlayersDB : public Base, public Gtk::StickyDialog

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
