/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "program_updated.h"

#include "ui.h"

#include "../../party/party.h"
#include "../../player/player.h"
#include "../../player/aiconfig.h"

#include <gtkmm/image.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/textview.h>
#include <gtkmm/texttag.h>

namespace UI_GTKMM_NS {

/** constructor
 **
 ** @param    parent        the parent object
 ** @param    old_version   last version
 **/
ProgramUpdated::ProgramUpdated(Base* const parent,
                               Version const& old_version) :
  Base(parent),
  Dialog("FreeDoko – " + _("Window::program updated")),
  old_version(old_version)
{
  this->ui->add_window(*this);

  this->set_icon(this->ui->icon);

  this->set_default_size(static_cast<int>(this->ui->logo->get_width() * 1.5),
                         static_cast<int>(this->ui->logo->get_height() * 2.5));
#ifdef POSTPONED
  this->get_window()->set_decorations(Gdk::DECOR_BORDER
                                      | Gdk::DECOR_RESIZEH
                                      | Gdk::DECOR_TITLE
                                      | Gdk::DECOR_MENU);
#endif

  { // action area
    { // reset ais button
      auto reset_ais_button = Gtk::manage(new Gtk::Button(_("Button::reset ais")));
      this->add_action_widget(*reset_ais_button, Gtk::RESPONSE_NONE);

      // signals
      reset_ais_button->signal_clicked().connect(sigc::mem_fun(*this, &ProgramUpdated::reset_ais));
    } // reset ais button
    add_close_button(*this);
  } // action area


  { // the image
    auto image = Gtk::manage(new Gtk::Image(parent->ui->logo));
    this->get_content_area()->pack_start(*image, Gtk::PACK_SHRINK);
  } // the image
  { // the text
    auto const text = Gtk::manage(new Gtk::TextView());
    this->text_buffer = text->get_buffer();
    {
      auto tag_title = this->text_buffer->create_tag("title");
      //tag_title->set_property("weight", Pango::WEIGHT_BOLD);
      tag_title->set_property("scale", Pango::SCALE_X_LARGE);
      tag_title->set_property("justification", Gtk::JUSTIFY_CENTER);

      auto tag_subtitle = this->text_buffer->create_tag("subtitle");
      tag_title->set_property("weight", Pango::WEIGHT_BOLD);
    }
    { // title
      this->text_buffer->insert_with_tag(this->text_buffer->begin(),
                                         "\n"
                                         // gettext: %s = version
                                         + _("Greeting::%s", ::version->number_to_string()),
                                         "title");
    } // title
    for (auto v = ::all_versions.rbegin();
         ( (v != ::all_versions.rend())
          && (this->old_version < **v) );
         ++v) {
      this->text_buffer->insert_with_tag(this->text_buffer->end(),
                                         "\n\n"
                                         // gettext: %s = version
                                         + _("ChangeLog::for %s", (*v)->number_to_string()),
                                         "subtitle");
      this->text_buffer->insert(this->text_buffer->end(),
                                "\n"
                                + _("ChangeLog::" + (*v)->number_to_string()));
    } // for (v \in ::all_versions)


    auto text_window = Gtk::manage(new Gtk::ScrolledWindow());
    text_window->add(*text);
    text_window->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    text->set_editable(false);
    text->set_wrap_mode(Gtk::WRAP_WORD);
    text->set_cursor_visible(false);

    this->get_content_area()->pack_end(*text_window);
  } // the text

  this->show_all_children();
} // ProgramUpdated::ProgramUpdated(Base* parent, Version old_version)

/** destructor
 **/
ProgramUpdated::~ProgramUpdated() = default;

/** reset the ais
 **/
void
ProgramUpdated::reset_ais()
{
  unsigned n = 0;
  for (auto& p : this->ui->party().players())
    if (dynamic_cast<Aiconfig*>(&p))
      dynamic_cast<Aiconfig&>(p).reset_to_hardcoded(n);

  this->hide();
} // void ProgramUpdated::reset_ais()

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
