/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "support.h"

#include "ui.h"

#include <gtkmm/image.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/textview.h>

namespace UI_GTKMM_NS {

/** constructor
 **
 ** @param    parent   the parent object
 **/
Support::Support(Base* const parent) :
  Base(parent),
  StickyDialog("FreeDoko – " + _("Window::support"), false)
{
  this->ui->add_window(*this);
  this->signal_realize().connect(sigc::mem_fun(*this, &Support::init));
} // Support::Support(Base* parent)

/** destruktor
 **/
Support::~Support() = default;

/** create all subelements
 **/
void
Support::init()
{
  this->set_icon(this->ui->icon);

  this->set_skip_taskbar_hint();

  this->text = Gtk::manage(new Gtk::TextView());

  this->text->get_buffer()->set_text(_("Version: %s",
                                       static_cast<string>(*::version))
                                     + "\n\n"
                                     + _("Support::support"));

  add_close_button(*this);

  this->set_default_size((this->ui->logo->get_width() * 3) / 2,
                         (this->ui->logo->get_height() * 5) / 2);
#ifdef POSTPONED
  this->get_window()->set_decorations(Gdk::DECOR_BORDER
                                      | Gdk::DECOR_RESIZEH
                                      | Gdk::DECOR_TITLE
                                      | Gdk::DECOR_MENU);
#endif

  { // the image
    auto image = Gtk::manage(new Gtk::Image(parent->ui->logo));
    this->get_content_area()->pack_start(*image, Gtk::PACK_SHRINK);
  } // the image
  { // the text
    auto text_window = Gtk::manage(new Gtk::ScrolledWindow());
    text_window->add(*this->text);
    text_window->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    this->text->set_editable(false);
    this->text->set_wrap_mode(Gtk::WRAP_WORD);
    this->text->set_cursor_visible(false);

    this->get_content_area()->pack_start(*text_window);
  } // the text

  this->show_all_children();
} // void Support::init()

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
