/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#ifdef USE_UI_GTKMM

#pragma once

#include "htin.h"
class Trick;

#include <pangomm/layout.h>

namespace UI_GTKMM_NS {

/**
 ** the graphic representation of a trickpile with the points
 **
 ** @todo   use of 'Gdk::Point' and 'Gdk::Rectangle'
 **/
class TrickPile : public HTIN {
  public:
    /**
     ** The status for checking changes (for redrawing)
     **/
    struct Status {
      unsigned size;
      vector<Team> teaminfo;
    }; // struct Status
    /**
     ** The geometry of the trickpile
     **/
    struct Geometry : public Table::Part::Geometry {
      function<int()> tricks_step_x;
      function<int()> tricks_step_y;
      function<int()> cards_step_x;
      function<int()> cards_step_y;
      function<int()> number_of_tricks;

      Outline outline() const override;
    }; // struct Geometry : public Table::Part::Geometry
  public:
    TrickPile(Table* table, Position position);
    ~TrickPile() override;

    bool changed() const override;
    Geometry geometry() const;
    Outline outline() const override;

    void draw(Cairo::RefPtr<::Cairo::Context> cr) override;
    void draw_tricks(Cairo::RefPtr<::Cairo::Context> cr);
    void draw_points(Cairo::RefPtr<::Cairo::Context> cr);

    ::Trick const* over_trick(int x, int y) const;

  private:
    Status current_status() const;
    Status last_status_;

  private: // unused
    TrickPile() = delete;
    TrickPile(TrickPile const&) = delete;
    TrickPile& operator=(TrickPile const&) = delete;
}; // class TrickPile : public HTIN

bool operator!=(TrickPile::Status const& lhs, TrickPile::Status const& rhs);

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
