/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#include "ui.dummy.h"

#ifndef POSTPONED
#ifndef RELEASE
#include <unistd.h>
#endif
#endif

#ifdef USE_UI_DUMMY_TEXT
#include "text/ui.h"
#endif // #ifdef USE_UI_DUMMY_TEXT
#ifdef USE_UI_DUMMY_GTKMM
#include "gtkmm/ui.h"
#endif // #ifdef USE_UI_DUMMY_GTKMM

#include "../party/party.h"
#include "../card/hand.h"
#include "../misc/setting.h"
#include "../os/bug_report.h"
#include "../os/bug_report_replay.h"

/** constructor
 **/
UI_Dummy::UI_Dummy() :
  UI(UIType::DUMMY)
{ }

/** the party is closed
 ** end the program
 **/
void
UI_Dummy::party_close()
{
  this->UI::party_close();
} // void UI_Dummy::party_close()

/** gets the reservations of the player 'player'
 ** Here is the first time, the hands can be shown (and should)
 **
 ** @param    player   the player that makes the reservation
 **
 ** @return   Reservation of the player 'player'
 **/
Reservation
UI_Dummy::get_reservation(Player const& player)
{ return player.reservation_get_default(); }

/** gets the card, the player wishes to play
 ** (don't remove it from the hand!)
 **
 ** @param    player   the player who shall play a card
 **
 ** @return   Card to be played
 **/
HandCard
UI_Dummy::get_card(Player const& player)
{ return player.hand().validcards(this->game().tricks().current()).highest_card(); }

/** returns which cards the player shifts
 **
 ** @param    player   the player who shifts the cards
 **
 ** @return   the cards that are to be shifted
 **/
HandCards
UI_Dummy::get_poverty_cards_to_shift(Player& player)
{ return HandCards(); }

/** returns whether 'player' accepts the shifted cards
 **
 ** @param    player   the player who shifts the cards
 ** @param    cardno   the number of shifted cards
 **
 ** @return   whether to accept the cards
 **/
bool
UI_Dummy::get_poverty_take_accept(Player const& player,
                                  unsigned const cardno)
{ return false; }

/** changes the cards from the poverty-player
 **
 ** @param    player   the player who has accepted the cards
 ** @param    cards   the cards that are given to the player
 **
 ** @return   the cards that are returned to the poverty-player
 **/
HandCards
UI_Dummy::get_poverty_exchanged_cards(Player& player, HandCards const& cards)
{ return HandCards(); }

/** genscher: -> result
 **
 ** @return   pointer to the new teammate,
 **   nullptr if the genscher is not to be announced
 **/
Player const*
UI_Dummy::get_genscher_partner()
{ return nullptr; } 

/** show the message
 **
 ** @param    message   message
 ** @param    type   type of the information
 ** @param    force_show   whether to force the showing of the massage (default: false)
 **/
void
UI_Dummy::information(string const& message,
		     InformationType const type,
                     bool const force_show)
{
  this->UI::information(message, type, force_show);
  cout << message << endl;
}

/** show the message and quit the program
 **
 ** @param    message   error message
 **/
void
UI_Dummy::error(string const& message)
{
  cerr << message << endl;
  if (::bug_report_replay != nullptr)
    cerr << "--\n"
      << _("replayed bug report: %s", ::bug_report_replay->filename()) << '\n';

  { // create bug report
    auto const filename = ::bug_report->report(message);

    // gettext: %s: filename, %s: contact
    cerr << _("BugReport::automatic creation, %s, %s", filename, CONTACT)
      << endl;
  } // create bug report

#ifdef ASSERTION_GENERATES_SEGFAULT
  {
    cerr << _("Creating segmentation fault.") << endl;
    SEGFAULT;
  }
#endif // #ifdef ASSERTION_GENERATES_SEGFAULT

  ::exit(EXIT_FAILURE);
} // void UI_Dummy::error(string message)
