/**
 **   Copyright (C) 2001 – 2018 by Diether Knof
 **
 **   This program is free software; you can redistribute it and/or 
 **   modify it under the terms of the GNU General Public License as 
 **   published by the Free Software Foundation; either version 2 of 
 **   the License, or (at your option) any later version.
 **
 **   This program is distributed in the hope that it will be useful,
 **   but WITHOUT ANY WARRANTY; without even the implied warranty of
 **   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **   GNU General Public License for more details. 
 **   You can find this license in the file 'gpl.txt'.
 **
 **   You should have received a copy of the GNU General Public License
 **   along with this program; if not, write to the Free Software
 **   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 **   MA  02111-1307  USA
 **
 **  Contact:
 **    Diether Knof dknof@posteo.de
 **/

#pragma once

#include <iosfwd>
#include <string>
#include <memory>

/**
 ** a date class
 **/
struct Date {
  public:
    // read the date from the stream
    static std::unique_ptr<Date> create(std::istream& istr);
    // read the date from the string
    static std::unique_ptr<Date> create(std::string const& s);

  public:
    Date();
    Date(int year, unsigned month, unsigned day) noexcept;
    explicit Date(std::string const& s);
    Date(Date const& date) = default;
    Date& operator=(Date const& date) = default;
    ~Date() = default;

    // convert in a string, p.e. 2006-10-11
    operator std::string const() const;

    int year;
    unsigned month;
    unsigned day;
}; // struct Date

bool operator==(Date const& lhs, Date const& rhs);
bool operator!=(Date const& lhs, Date const& rhs);
bool operator<(Date const& lhs, Date const& rhs);
bool operator>(Date const& lhs, Date const& rhs);
bool operator<=(Date const& lhs, Date const& rhs);
bool operator>=(Date const& lhs, Date const& rhs);

std::ostream& operator<<(std::ostream& ostr, Date const& date);
