/**********************************************************************
 *
 *   Copyright (C) 2001 – 2018 by Diether Knof
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *   Contact:
 *     Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include <string>
using std::string;

namespace Parser {
// Parsen von mehreren verschiedenen Argumenten (Rekursion)
template<typename... Args>	
  bool parse(string const& text, char const* fixtext, Args&... args);
template<typename T, typename... Args>	
  bool parse(string const& text, T& param, Args&... args);
bool parse(string& text);
// Hilfsmethoden
bool parse_(string& text, unsigned& n);
bool parse_(string& text, string& remaining_text);
bool parse_(string& text, char const* fixtext);

}; // namespace Parser

/** Prüft, ob text mit fixtext beginnt. Wenn ja, wird weiter geparst, ansonesten nicht.
 ** Gibt zurück, ob die angegebenen Argumente gepasst haben.
 ** Wenn alle Argumente passen, werden die Parameter auf den entsprechenden Wert gesetzt.
 **/
template<typename... Args>
bool
Parser::parse(string const& text, char const* fixtext, Args&... args)
{
  auto text2 = text;
  if (!parse_(text2, fixtext)) 
    return false;
  if (!parse(text2, args...))
    return false;
  return true;
} // template<typename... Args> bool Eingabe::parse(string text, char const* fixtext, Args... args)

/** Prüft, ob text mit Typ T beginnt. Wenn ja, wird weiter geparst, ansonesten nicht.
 ** Gibt zurück, ob die angegebenen Argumente gepasst haben.
 ** Wenn alle Argumente passen, werden die Parameter auf den entsprechenden Wert gesetzt.
 **/
template<typename T, typename... Args>
bool
Parser::parse(string const& text, T& param, Args&... args)
{
  auto text2 = text;
  T p;
  if (!parse_(text2, p)) 
    return false;
  if (!parse(text2, args...))
    return false;
  param = std::move(p);
  return true;
} // template<typename T, typename... Args> bool Eingabe::parse(string text, T& param, Args... args)
